library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library work;
use work.oscilloscope_pck.all;

entity controller is
	port (
		clk			: in std_logic;
		reset			: in std_logic;

		display_vector		: in std_logic;
		display_roll		: in std_logic;

		-- Display RAM signals
		display_ram_wr_addr	: out std_logic_vector(10 downto 0);
		display_ram_datain	: out std_logic_vector(36 downto 0);
		display_ram_wr		: out std_logic;
		vblank			: in std_logic;

		-- Sampler RAM signals
		sampler_ram_rd_addr	: out std_logic_vector(10 downto 0);
		sampler_ram_dataout	: in std_logic_vector(31 downto 0);
		sampler_ram_wr_addr	: in std_logic_vector(10 downto 0);

		ch0_gain		: in std_logic_vector(2 downto 0);
		ch1_gain		: in std_logic_vector(2 downto 0);
		ch0_offset		: in std_logic_vector(10 downto 0);
		ch1_offset		: in std_logic_vector(10 downto 0);

		sampler_rdy		: in std_logic;
		copy_rdy		: out std_logic
	);
end controller;

architecture rtl of controller is

signal s_sampler_ram_rd_addr : std_logic_vector(10 downto 0);
signal s_display_ram_wr_addr : std_logic_vector(10 downto 0);
signal s_display_ram_wr : std_logic;
signal s_vblank : std_logic;

signal s_ch0_sample_min0 : std_logic_vector(8 downto 0);
signal s_ch0_sample_max0 : std_logic_vector(8 downto 0);
signal s_ch1_sample_min0 : std_logic_vector(8 downto 0);
signal s_ch1_sample_max0 : std_logic_vector(8 downto 0);

signal s_ch0_sample_min1 : std_logic_vector(8 downto 0);
signal s_ch0_sample_max1 : std_logic_vector(8 downto 0);
signal s_ch1_sample_min1 : std_logic_vector(8 downto 0);
signal s_ch1_sample_max1 : std_logic_vector(8 downto 0);

signal s_ch0_display_min : std_logic_vector(8 downto 0);
signal s_ch0_display_max : std_logic_vector(8 downto 0);
signal s_ch1_display_min : std_logic_vector(8 downto 0);
signal s_ch1_display_max : std_logic_vector(8 downto 0);

type state_type is (idle, copy1_0, copy1_1, copy2, copy3, copy4, copy5);
signal state : state_type;

signal s_sampler_ram_wr_addr : std_logic_vector(10 downto 0);
signal s_sampler_rdy : std_logic;

begin

	clk_domain_sync: process(clk, reset)
	variable v_sampler_ram_wr_addr : std_logic_vector(10 downto 0);
	variable v_sampler_rdy : std_logic;
	begin
		if reset = '1' then
			v_sampler_ram_wr_addr := (others => '0');
			v_sampler_rdy := '0';
		elsif clk'event and clk = '1' then
			s_sampler_ram_wr_addr <= v_sampler_ram_wr_addr;
			v_sampler_ram_wr_addr := sampler_ram_wr_addr;
			s_sampler_rdy <= v_sampler_rdy;
			v_sampler_rdy := sampler_rdy;
		end if;
	end process;
	
	
	offset_gain: process(clk, reset)
	variable v_ch0_min : std_logic_vector(11 downto 0);
	variable v_ch0_max : std_logic_vector(11 downto 0);
	variable v_ch1_min : std_logic_vector(11 downto 0);
	variable v_ch1_max : std_logic_vector(11 downto 0);
	begin
		if reset = '1' then
		elsif clk'event and clk = '1' then
			-- signed waveforms and sign extension to 12 bits
			for i in 11 downto 7 loop
				v_ch0_min(i) := not sampler_ram_dataout(7);
				v_ch0_max(i) := not sampler_ram_dataout(15);
				v_ch1_min(i) := not sampler_ram_dataout(23);
				v_ch1_max(i) := not sampler_ram_dataout(31);
			end loop;
			v_ch0_min(6 downto 0) := sampler_ram_dataout(6 downto 0);
			v_ch0_max(6 downto 0) := sampler_ram_dataout(14 downto 8);
			v_ch1_min(6 downto 0) := sampler_ram_dataout(22 downto 16);
			v_ch1_max(6 downto 0) := sampler_ram_dataout(30 downto 24);

			-- gain and offset, result on 12 bits
			if ch0_gain = "000" then	-- /4
				v_ch0_min := (v_ch0_min(11) & v_ch0_min(11) & v_ch0_min(11 downto 2)) + ("0000000" & v_ch0_min(1)) + (ch0_offset(10) & ch0_offset);
				v_ch0_max := (v_ch0_max(11) & v_ch0_max(11) & v_ch0_max(11 downto 2)) + ("0000000" & v_ch0_max(1)) + (ch0_offset(10) & ch0_offset);
			elsif ch0_gain = "001" then	-- /2
				v_ch0_min := (v_ch0_min(11) & v_ch0_min(11 downto 1)) + ("0000000" & v_ch0_min(0)) + (ch0_offset(10) & ch0_offset);
				v_ch0_max := (v_ch0_max(11) & v_ch0_max(11 downto 1)) + ("0000000" & v_ch0_max(0)) + (ch0_offset(10) & ch0_offset);
			elsif ch0_gain = "010" then	-- *1
				v_ch0_min := v_ch0_min + (ch0_offset(10) & ch0_offset);
				v_ch0_max := v_ch0_max + (ch0_offset(10) & ch0_offset);
			elsif ch0_gain = "011" then	-- *2
				v_ch0_min := (v_ch0_min(10 downto 0) & '0') + (ch0_offset(10) & ch0_offset);
				v_ch0_max := (v_ch0_max(10 downto 0) & '0') + (ch0_offset(10) & ch0_offset);
			elsif ch0_gain = "100" then	-- *4
				v_ch0_min := (v_ch0_min(9 downto 0) & "00") + (ch0_offset(10) & ch0_offset);
				v_ch0_max := (v_ch0_max(9 downto 0) & "00") + (ch0_offset(10) & ch0_offset);
			end if;
			if ch1_gain = "000" then	-- /4
				v_ch1_min := (v_ch1_min(11) & v_ch1_min(11) & v_ch1_min(11 downto 2)) + ("0000000" & v_ch1_min(1)) + (ch1_offset(10) & ch1_offset);
				v_ch1_max := (v_ch1_max(11) & v_ch1_max(11) & v_ch1_max(11 downto 2)) + ("0000000" & v_ch1_max(1)) + (ch1_offset(10) & ch1_offset);
			elsif ch1_gain = "001" then	-- /2
				v_ch1_min := (v_ch1_min(11) & v_ch1_min(11 downto 1)) + ("0000000" & v_ch1_min(0)) + (ch1_offset(10) & ch1_offset);
				v_ch1_max := (v_ch1_max(11) & v_ch1_max(11 downto 1)) + ("0000000" & v_ch1_max(0)) + (ch1_offset(10) & ch1_offset);
			elsif ch1_gain = "010" then	-- *1
				v_ch1_min := v_ch1_min + (ch1_offset(10) & ch1_offset);
				v_ch1_max := v_ch1_max + (ch1_offset(10) & ch1_offset);
			elsif ch1_gain = "011" then	-- *2
				v_ch1_min := (v_ch1_min(10 downto 0) & '0') + (ch1_offset(10) & ch1_offset);
				v_ch1_max := (v_ch1_max(10 downto 0) & '0') + (ch1_offset(10) & ch1_offset);
			elsif ch1_gain = "100" then	-- *4
				v_ch1_min := (v_ch1_min(9 downto 0) & "00") + (ch1_offset(10) & ch1_offset);
				v_ch1_max := (v_ch1_max(9 downto 0) & "00") + (ch1_offset(10) & ch1_offset);
			end if;

			-- unsign
			v_ch0_min(11) := not v_ch0_min(11);
			v_ch0_max(11) := not v_ch0_max(11);
			v_ch1_min(11) := not v_ch1_min(11);
			v_ch1_max(11) := not v_ch1_max(11);

			-- limiter to range -256..255 -> 0..511, 0V=256
			if v_ch0_min > 2048+255 then
				s_ch0_sample_min0 <= conv_std_logic_vector(2048+511, 9);
			elsif v_ch0_min < 2048-256 then
				s_ch0_sample_min0 <= conv_std_logic_vector(2048, 9);
			else
				s_ch0_sample_min0 <= v_ch0_min(8 downto 0)+256;
			end if;
			if v_ch0_max > 2048+255 then
				s_ch0_sample_max0 <= conv_std_logic_vector(2048+511, 9);
			elsif v_ch0_max < 2048-256 then
				s_ch0_sample_max0 <= conv_std_logic_vector(2048, 9);
			else
				s_ch0_sample_max0 <= v_ch0_max(8 downto 0)+256;
			end if;
			if v_ch1_min > 2048+255 then
				s_ch1_sample_min0 <= conv_std_logic_vector(2048+511, 9);
			elsif v_ch1_min < 2048-256 then
				s_ch1_sample_min0 <= conv_std_logic_vector(2048, 9);
			else
				s_ch1_sample_min0 <= v_ch1_min(8 downto 0)+256;
			end if;
			if v_ch1_max > 2048+255 then
				s_ch1_sample_max0 <= conv_std_logic_vector(2048+511, 9);
			elsif v_ch1_max < 2048-256 then
				s_ch1_sample_max0 <= conv_std_logic_vector(2048, 9);
			else
				s_ch1_sample_max0 <= v_ch1_max(8 downto 0)+256;
			end if;
		end if;
	end process;


	process(clk, reset)
	variable v_avg : std_logic_vector(9 downto 0);
	variable v_display_vector : std_logic;
	begin
		if reset = '1' then
			s_vblank <= '0';
			s_sampler_ram_rd_addr <= (others => '0');
			s_display_ram_wr_addr <= (others => '0');
			s_display_ram_wr <= '0';
			copy_rdy <= '0';
			s_ch0_sample_min1 <= (others => '0');
			s_ch0_sample_max1 <= (others => '0');
			s_ch1_sample_min1 <= (others => '0');
			s_ch1_sample_max1 <= (others => '0');
			s_ch0_display_min <= (others => '0');
			s_ch0_display_max <= (others => '0');
			s_ch1_display_min <= (others => '0');
			s_ch1_display_max <= (others => '0');
			state <= idle;
			v_avg := (others => '0');
			v_display_vector := '0';
		elsif clk'event and clk = '1' then

			s_vblank <= vblank;

			case state is
			when idle =>
				if vblank = '1' and s_vblank = '0' and s_sampler_rdy = '1' then
--				if s_sampler_rdy = '1' then
					if display_roll = '1' then
						s_sampler_ram_rd_addr <= s_sampler_ram_wr_addr - waveform_length - 2; -- 779;
					else
						s_sampler_ram_rd_addr <= s_sampler_ram_wr_addr - waveform_length - 0; --901; -- (1024-779)/2 + 779 = 901  - TBD !!!!!!!! - this should be relative to trigger delay
					end if;
					v_display_vector := display_vector;
					state <= copy1_0;
				end if;
				s_display_ram_wr_addr <= (others => '0');
				s_display_ram_wr <= '0';
				copy_rdy <= '0';

			when copy1_0 =>			-- first address to sampler RAM is valid
				s_sampler_ram_rd_addr <= s_sampler_ram_rd_addr + 1;
				state <= copy1_1;

			when copy1_1 =>			-- data on sampler RAM output
				s_sampler_ram_rd_addr <= s_sampler_ram_rd_addr + 1;
				state <= copy2;

			when copy2 =>				-- data on output of offset_gain process
				s_sampler_ram_rd_addr <= s_sampler_ram_rd_addr + 1;
				s_ch0_sample_min1 <= s_ch0_sample_min0;
				s_ch0_sample_max1 <= s_ch0_sample_max0;
				s_ch1_sample_min1 <= s_ch1_sample_min0;
				s_ch1_sample_max1 <= s_ch1_sample_max0;
				state <= copy3;

			when copy3 =>
				s_sampler_ram_rd_addr <= s_sampler_ram_rd_addr + 1;
				if s_display_ram_wr = '1' then
					s_display_ram_wr_addr <= s_display_ram_wr_addr + 1;
				end if;
				s_display_ram_wr <= '1';
				if v_display_vector = '0' then
					s_ch0_display_min <= s_ch0_sample_min1;
					s_ch0_display_max <= s_ch0_sample_max1;
					s_ch1_display_min <= s_ch1_sample_min1;
					s_ch1_display_max <= s_ch1_sample_max1;
					s_ch0_sample_min1 <= s_ch0_sample_min0;
					s_ch0_sample_max1 <= s_ch0_sample_max0;
					s_ch1_sample_min1 <= s_ch1_sample_min0;
					s_ch1_sample_max1 <= s_ch1_sample_max0;
				else
					if s_ch0_sample_min0 > s_ch0_sample_max1 then
						v_avg := ('0' & s_ch0_sample_max1) + ('0' & s_ch0_sample_min0);
						s_ch0_display_max <= v_avg(9 downto 1);
						s_ch0_sample_min1 <= v_avg(9 downto 1) + 1;
					else
						s_ch0_display_max <= s_ch0_sample_max1;
						s_ch0_sample_min1 <= s_ch0_sample_min0;
					end if;
					if s_ch0_sample_max0 < s_ch0_sample_min1 then
						v_avg := ('0' & s_ch0_sample_min1) + ('0' & s_ch0_sample_max0);
						s_ch0_display_min <= v_avg(9 downto 1) + 1;
						s_ch0_sample_max1 <= v_avg(9 downto 1);
					else
						s_ch0_display_min <= s_ch0_sample_min1;
						s_ch0_sample_max1 <= s_ch0_sample_max0;
					end if;
					if s_ch1_sample_min0 > s_ch1_sample_max1 then
						v_avg := ('0' & s_ch1_sample_max1) + ('0' & s_ch1_sample_min0);
						s_ch1_display_max <= v_avg(9 downto 1);
						s_ch1_sample_min1 <= v_avg(9 downto 1) + 1;
					else
						s_ch1_display_max <= s_ch1_sample_max1;
						s_ch1_sample_min1 <= s_ch1_sample_min0;
					end if;
					if s_ch1_sample_max0 < s_ch1_sample_min1 then
						v_avg := ('0' & s_ch1_sample_min1) + ('0' & s_ch1_sample_max0);
						s_ch1_display_min <= v_avg(9 downto 1) + 1;
						s_ch1_sample_max1 <= v_avg(9 downto 1);
					else
						s_ch1_display_min <= s_ch1_sample_min1;
						s_ch1_sample_max1 <= s_ch1_sample_max0;
					end if;
				
				end if;
				if conv_integer(s_display_ram_wr_addr) = waveform_length + 1 then --777 then
					state <= copy4;
				end if;
			when copy4 =>
				s_display_ram_wr_addr <= s_display_ram_wr_addr + 1;
				s_display_ram_wr <= '1';
				s_ch0_display_min <= s_ch0_sample_min1;
				s_ch0_display_max <= s_ch0_sample_max1;
				s_ch1_display_min <= s_ch1_sample_min1;
				s_ch1_display_max <= s_ch1_sample_max1;
				state <= copy5;
				copy_rdy <= '1';
			when copy5 =>
				state <= idle;
				s_display_ram_wr <= '0';
				copy_rdy <= '0';
			when others =>
				state <= idle;
			end case;
		end if;
	end process;

	sampler_ram_rd_addr <= s_sampler_ram_rd_addr;
	display_ram_wr_addr <= s_display_ram_wr_addr;
	display_ram_datain <= '1' & s_ch1_display_max & s_ch1_display_min & s_ch0_display_max & s_ch0_display_min;
	display_ram_wr <= s_display_ram_wr;

end rtl;





