-- clk input					: 50MHz
-- clk_main						: 40..45MHz
-- clk_lcd (clk_main*7)		: 280..315MHz
-- clk_sampler (internal)	: 160..200MHz - 4*(or 5)*clk_main
-- clk_adc (output)			: 160..200MHz - frequency is same as of clk_sampler, but phase-shifted - PHASE_SHIFT parameter of dcm3_clk_adc_driver must be tuned

-- Note: according to the datasheet (DS312) the DFS on the FPGA of the Spartan 3E starter board (XC3S500E, speed grade 4, stepping 0) can not output frequencies between 90 and 220MHz, however it does on my board.

library ieee;
use ieee.std_logic_1164.ALL;
use ieee.numeric_std.ALL;
library UNISIM;
use UNISIM.Vcomponents.ALL;

entity dcm is
	port (
		clk			: in std_logic;
		reset			: in std_logic;
		clk_main		: out std_logic;
		clk_lcd		: out std_logic;
		clk_sampler	: out std_logic;
		clk_adc		: out std_logic;
		locked		: out std_logic;
		status0		: out std_logic_vector (7 downto 0);
		status1		: out std_logic_vector (7 downto 0);
		status2		: out std_logic_vector (7 downto 0);
		status3		: out std_logic_vector (7 downto 0)
	);
end dcm;

architecture bhv of dcm is

signal clk_ibufg : std_logic;

signal clk0 : std_logic;
signal clk1 : std_logic;
signal clk2 : std_logic;
signal clk3 : std_logic;

signal clk0_bufg : std_logic;
signal clk1_bufg : std_logic;
signal clk2_bufg : std_logic;
signal clk3_bufg : std_logic;

signal clkfx0 : std_logic;
signal clkfx1 : std_logic;
signal clkfx2 : std_logic;
signal clkfx3 : std_logic;
signal clkfx3_n : std_logic;

signal clkfx0_bufg : std_logic;
signal clkfx1_bufg : std_logic;
signal clkfx2_bufg : std_logic;
signal clkfx3_bufg : std_logic;
signal clkfx3_n_bufg : std_logic;

signal locked0	: std_logic;
signal locked1	: std_logic;
signal locked2	: std_logic;
signal locked3	: std_logic;

signal logic_0 : std_logic;

begin

	clk_input_buffer : IBUFG port map (I => clk, O => clk_ibufg);

	clk0_output_buffer : BUFG port map (I => clk0, O => clk0_bufg);
	clk1_output_buffer : BUFG port map (I => clk1, O => clk1_bufg);
	clk2_output_buffer : BUFG port map (I => clk2, O => clk2_bufg);
--	clk3_output_buffer : BUFG port map (I => clk3, O => clk3_bufg);
	clk3_bufg <= clk3;

	clkfx0_output_buffer : BUFG port map (I => clkfx0, O => clkfx0_bufg);
	clkfx1_output_buffer : BUFG port map (I => clkfx1, O => clkfx1_bufg);
	clkfx2_output_buffer : BUFG port map (I => clkfx2, O => clkfx2_bufg);
	clkfx3_output_buffer : BUFG port map (I => clkfx3, O => clkfx3_bufg);
	clkfx3_n_output_buffer : BUFG port map (I => clkfx3_n, O => clkfx3_n_bufg);

	locked <= locked0 and locked1 and locked2 and locked3;

	logic_0 <= '0';

	clk_main	<= clk1_bufg; --clkfx0_bufg;
	clk_lcd		<= clkfx1_bufg;
	clk_sampler	<= clkfx2_bufg;

	clk_adc_driver : ODDR2
	generic map (
		DDR_ALIGNMENT => "NONE", 	-- Sets output alignment to "NONE", "C0", "C1" 
		INIT => '0', 					-- Sets initial state of the Q output to '0' or '1'
		SRTYPE => "SYNC"  			-- Specifies "SYNC" or "ASYNC" set/reset
	)
	port map (
		Q  => clk_adc,
		C0 => clkfx3_bufg,
		C1 => clkfx3_n_bufg,
		CE => '1',
		D0 => '0',
		D1 => '1',
		R  => '0',
		S  => '0'		
	);

	dcm0_clk_main: DCM_SP
	generic map(
		CLK_FEEDBACK => "1X",
		CLKDV_DIVIDE => 2.0,
		CLKFX_DIVIDE => 5,
		CLKFX_MULTIPLY => 4,						-- 9/10->45MHz, 4/5->40MHz, 2/3->33.33MHz
		CLKIN_DIVIDE_BY_2 => FALSE,
		CLKIN_PERIOD => 20.000,
		CLKOUT_PHASE_SHIFT => "NONE",
		DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS",
		DFS_FREQUENCY_MODE => "HIGH",
		DLL_FREQUENCY_MODE => "LOW",
		DUTY_CYCLE_CORRECTION => TRUE,
--		FACTORY_JF => x"C080",					-- x"8080" ???
		PHASE_SHIFT => 0,
		STARTUP_WAIT => FALSE
	)
	port map (
		CLKFB		=> clk0_bufg,
		CLKIN		=> clk_ibufg,
		DSSEN		=> logic_0,
		PSCLK		=> logic_0,
		PSEN		=> logic_0,
		PSINCDEC	=> logic_0,
		RST		=> reset,
		CLKFX		=> clkfx0,
--		CLKFX180	=> clkfx0_n,
		CLK0		=> clk0,
		LOCKED	=> locked0,
		STATUS	=> status0
	);

	dcm1_clk_lcd: DCM_SP
	generic map(
		CLK_FEEDBACK => "1X",
		CLKDV_DIVIDE => 2.0,
		CLKFX_DIVIDE => 1,
		CLKFX_MULTIPLY => 7,
		CLKIN_DIVIDE_BY_2 => FALSE,
		CLKIN_PERIOD => 25.000,
		CLKOUT_PHASE_SHIFT => "NONE",
		DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS",
		DFS_FREQUENCY_MODE => "HIGH",
		DLL_FREQUENCY_MODE => "LOW",
		DUTY_CYCLE_CORRECTION => TRUE,
--		FACTORY_JF => x"C080",					-- x"8080" ???
		PHASE_SHIFT => 0,
		STARTUP_WAIT => FALSE
	)
	port map (
		CLKFB		=> clk1_bufg,
		CLKIN		=> clkfx0_bufg,				-- output of from dcm0
		DSSEN		=> logic_0,
		PSCLK		=> logic_0,
		PSEN		=> logic_0,
		PSINCDEC	=> logic_0,
		RST		=> reset,
		CLKFX		=> clkfx1,
--		CLKFX180	=> clkfx1_n,
		CLK0		=> clk1,
		LOCKED	=> locked1,
		STATUS	=> status1
	);

	dcm2_clk_sampler: DCM_SP
	generic map(
		CLK_FEEDBACK => "1X",
		CLKDV_DIVIDE => 2.0,
		CLKFX_DIVIDE => 1,
		CLKFX_MULTIPLY => 4,						-- 4*40=160MHz, 4*45=180MHz
		CLKIN_DIVIDE_BY_2 => FALSE,
		CLKIN_PERIOD => 25.000,
		CLKOUT_PHASE_SHIFT => "NONE",
		DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS",
		DFS_FREQUENCY_MODE => "HIGH",
		DLL_FREQUENCY_MODE => "LOW",
		DUTY_CYCLE_CORRECTION => TRUE,
--		FACTORY_JF => x"C080",					-- x"8080" ???
		PHASE_SHIFT => 0,
		STARTUP_WAIT => FALSE
	)
	port map (
		CLKFB		=> clk2_bufg,
		CLKIN		=> clkfx0_bufg,				-- output of from dcm0
		DSSEN		=> logic_0,
		PSCLK		=> logic_0,
		PSEN		=> logic_0,
		PSINCDEC	=> logic_0,
		RST		=> reset,
		CLKFX		=> clkfx2,
--		CLKFX180	=> clkfx2_n,
		CLK0		=> clk2,
		LOCKED	=> locked2,
		STATUS	=> status2
	);

	dcm3_clk_adc_driver: DCM_SP
	generic map(
		CLK_FEEDBACK => "1X",
		CLKDV_DIVIDE => 2.0,
		CLKFX_DIVIDE => 1,
		CLKFX_MULTIPLY => 4,						-- 4*40=160MHz, 4*45=180MHz
		CLKIN_DIVIDE_BY_2 => FALSE,
		CLKIN_PERIOD => 25.000,
		CLKOUT_PHASE_SHIFT => "FIXED",
		DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS",
		DFS_FREQUENCY_MODE => "HIGH",
		DLL_FREQUENCY_MODE => "LOW",
		DUTY_CYCLE_CORRECTION => TRUE,
--		FACTORY_JF => x"C080",					-- x"8080" ???
		PHASE_SHIFT => 20,						-- if the design is modified anywhere then tune this and compile again until the phase of the ADC sample is acceptable (timing depends on placement)
		STARTUP_WAIT => FALSE
	)
	port map (
		CLKFB		=> clk3_bufg,
		CLKIN		=> clkfx0_bufg,				-- output of from dcm0
		DSSEN		=> logic_0,
		PSCLK		=> logic_0,
		PSEN		=> logic_0,
		PSINCDEC	=> logic_0,
		RST		=> reset,
		CLKFX		=> clkfx3,
		CLKFX180	=> clkfx3_n,
		CLK0		=> clk3,
		LOCKED	=> locked3,
		STATUS	=> status3
	);

end bhv;
