library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_unsigned.ALL;

library UNISIM;
use UNISIM.VComponents.all;

entity lcd_driver is
	port (
		clk_main	: in std_logic;
		clk_lcd	: in std_logic;
		reset		: in std_logic;

		red		: in std_logic;
		green		: in std_logic;
		blue		: in std_logic;
		hsync		: in std_logic;
		vsync		: in std_logic;
		de			: in std_logic;

		dout0p	: out std_logic;
		dout0n	: out std_logic;
		dout1p	: out std_logic;
		dout1n	: out std_logic;
		dout2p	: out std_logic;
		dout2n	: out std_logic;
		clkoutp	: out std_logic;
		clkoutn	: out std_logic
	);
end lcd_driver;

architecture rtl of lcd_driver is

signal s_reg0			: std_logic_vector(6 downto 0);
signal s_reg1 			: std_logic_vector(6 downto 0);
signal s_reg2 			: std_logic_vector(6 downto 0);
signal s_reg3 			: std_logic_vector(6 downto 0);

signal s_de 			: std_logic;
signal s_de0 			: std_logic;
signal s_de1 			: std_logic;

signal s_vsync  		: std_logic;
signal s_vsync0 		: std_logic;
signal s_vsync1	 	: std_logic;

signal s_hsync 		: std_logic;
signal s_hsync0		: std_logic;
signal s_hsync1 		: std_logic;

signal s_load 			: std_logic;
signal s_load0			: std_logic;
signal s_load1			: std_logic;
signal s_load2			: std_logic;
signal s_load_reg		: std_logic;

signal s_red 			: std_logic_vector(5 downto 0);
signal s_red0			: std_logic_vector(5 downto 0);
signal s_red1			: std_logic_vector(5 downto 0);

signal s_green			: std_logic_vector(5 downto 0);
signal s_green0		: std_logic_vector(5 downto 0);
signal s_green1		: std_logic_vector(5 downto 0);

signal s_blue			: std_logic_vector(5 downto 0);
signal s_blue0			: std_logic_vector(5 downto 0);
signal s_blue1			: std_logic_vector(5 downto 0);

signal s_hsync_cnt	: std_logic_vector(10 downto 0);
signal s_vsync_cnt	: std_logic_vector(9 downto 0);

signal s_clk_lo		: std_logic;
signal s_clk_hi 		: std_logic;
signal s_dcm_locked	: std_logic;
signal s_reset 		: std_logic;

begin

	dout0_driver: OBUFDS
	port map(
		O  => dout0p,
		OB	=> dout0n,
		I  => s_reg0(6)
	);

	dout1_driver: OBUFDS
	port map(
		O  => dout1p,
		OB	=> dout1n,
		I  => s_reg1(6)
	);

	dout2_driver: OBUFDS
	port map(
		O  => dout2p,
		OB	=> dout2n,
		I  => s_reg2(6)
	);

	clkout_driver: OBUFDS
	port map(
		O  => clkoutp,
		OB	=> clkoutn,
		I  => s_reg3(6)
	);

	load_shift_registers: process(clk_lcd, reset)
	begin
		if reset = '1' then
			s_reg0 <= (others => '0');
			s_reg1 <= (others => '0');
			s_reg2 <= (others => '0');
			s_reg3 <= (others => '0');
		elsif clk_lcd'event and clk_lcd = '1' then
			if s_load_reg = '1' then
				s_reg3 <= "1100011";										-- clkout
				s_reg2(6) <= s_de1;
				s_reg2(5) <= s_vsync1;
				s_reg2(4) <= s_hsync1;
				s_reg2(3 downto 0) <= s_blue1(5 downto 2);
				s_reg1(6 downto 5) <= s_blue1(1 downto 0);
				s_reg1(4 downto 0) <= s_green1(5 downto 1);
				s_reg0(6) <= s_green1(0);
				s_reg0(5 downto 0) <= s_red1(5 downto 0);
			else
				s_reg3(6 downto 1) <= s_reg3(5 downto 0);
				s_reg3(0) <= '0';
				s_reg2(6 downto 1) <= s_reg2(5 downto 0);
				s_reg2(0) <= '0';
				s_reg1(6 downto 1) <= s_reg1(5 downto 0);
				s_reg1(0) <= '0';
				s_reg0(6 downto 1) <= s_reg0(5 downto 0);
				s_reg0(0) <= '0';
			end if;
		end if;
	end process;

	register_load_signal: process(clk_lcd, reset)
	begin
		if reset = '1' then
			s_load2 <= '0';
			s_load_reg <= '0';
		elsif clk_lcd'event and clk_lcd = '1' then
			if s_load1 /= s_load2 then			-- s_load changes state in every period of the low speed clk (could count to 7 instead)
				s_load_reg <= '1';
			else
				s_load_reg <= '0';
			end if;
			s_load2 <= s_load1;
		end if;
	end process;

	clk_domain_sync: process(clk_lcd, reset)
	begin
		if reset = '1' then
			s_red0	<= (others => '0');
			s_green0	<= (others => '0');
			s_blue0	<= (others => '0');

			s_red1	<= (others => '0');
			s_green1	<= (others => '0');
			s_blue1	<= (others => '0');

			s_de0 	<= '0';
			s_hsync0 <= '0';
			s_vsync0 <= '0';
			s_load0 	<= '0';

			s_de1 	<= '0';
			s_hsync1 <= '0';
			s_vsync1 <= '0';
			s_load1 	<= '0';
		elsif clk_lcd'event and clk_lcd = '1' then
			s_red0 	<= s_red;
			s_green0 <= s_green;
			s_blue0 	<= s_blue;

			s_red1 	<= s_red0;
			s_green1 <= s_green0;
			s_blue1 	<= s_blue0;

			s_de0 	<= s_de;
			s_hsync0 <= s_hsync;
			s_vsync0 <= s_vsync;
			s_load0 	<= s_load;

			s_de1 	<= s_de0;
			s_hsync1 <= s_hsync0;
			s_vsync1 <= s_vsync0;
			s_load1 	<= s_load0;
		end if;
	end process;

	process(clk_main, reset)
	begin
		if reset = '1' then
			s_load <= '0';
			s_hsync <= '0';
			s_vsync <= '0';
			s_de <= '0';
			s_red <= (others => '0');
			s_green <= (others => '0');
			s_blue <= (others => '0');
		elsif clk_main'event and clk_main = '1' then
			s_load <= not s_load;
			s_hsync <= not hsync;
			s_vsync <= not vsync;
			s_de <= de;
			for i in 0 to 5 loop
				s_red(i)   <= red;
				s_green(i) <= green;
				s_blue(i)  <= blue;
			end loop;
		end if;
	end process;

end rtl;

