library ieee;
use ieee.std_logic_1164.all;

package oscilloscope_pck is

-- Waveform display dimensions
constant x0  : integer := 48;				-- waveform area top left corner x
constant y0  : integer := 32;				-- waveform area top left corner y
constant dx  : integer := 80;				-- number of pixels per horizontal div = number of samples in 500ns at the sampling frequency
constant dy  : integer := 64;				-- number of pixels per vertical div
constant nx  : integer := 15;				-- number of horizontal divs
constant ny  : integer := 10;				-- number of vertical divs
constant waveform_length : integer := nx*dx;   ---2; --- waveform length

-- Display timings
constant c_hres : integer := 1280;			-- horizontal resolution
constant c_hss  : integer := 1380;			-- horizontal sync start in pixels
constant c_hse  : integer := 1382;			-- horizontal sync end in pixels
constant c_hend : integer := 1399;			-- horizontal end of cycle in pixels

constant c_vres : integer := 800;			-- vertical resolution
constant c_vss  : integer := 810; 			-- vertical sync start in lines
constant c_vse  : integer := 811;			-- vertical sync end in lines
constant c_vend : integer := 815;			-- vertical end of cycle in lines

constant c_vbs  : integer := 802;			-- vertical blanking start in lines
constant c_vbe  : integer := 812;			-- vertical blanking end in lines

-- Text area
constant c_txt_x0 : integer := 4;			-- horizontal start coordinate of text area
constant c_txt_y0 : integer := 4;			-- vertical   start coordinate of text area
constant c_txt_dx : integer := 12;			-- horizontal size of characters
constant c_txt_dy : integer := 24;			-- vertical   size of characters
constant c_txt_nx : integer := 106;			-- horizontal number of characters
constant c_txt_ny : integer := 33;			-- vertical   number of characters

component display_driver
	port (
		clk    : in std_logic;
		reset  : in std_logic;

		wave_addr  : out std_logic_vector(10 downto 0);
		wave_data  : in std_logic_vector(36 downto 0);

		txt_addr   : out std_logic_vector(12 downto 0);
		txt_data   : in std_logic_vector(9 downto 0);

		vblank : out std_logic;		-- vertical blanking/retrace; RAM can be updated when '1'

		ch0_en : in std_logic;
		ch1_en : in std_logic;
		ch0_offset : in std_logic_vector(10 downto 0);
		ch1_offset : in std_logic_vector(10 downto 0);

		red    : out std_logic;
		green  : out std_logic;
		blue   : out std_logic;
		hsync  : out std_logic;
		vsync  : out std_logic;
		de     : out std_logic;

		mouse_x	: in std_logic_vector(10 downto 0);
		mouse_y	: in std_logic_vector(9 downto 0);

		trigger_level       : in std_logic_vector(9 downto 0);
		trigger_delay       : in std_logic_vector(10 downto 0);

		cursor_en     : in std_logic;
		cursor_level1 : in std_logic_vector(9 downto 0);
		cursor_level2 : in std_logic_vector(9 downto 0);
		cursor_delay1 : in std_logic_vector(10 downto 0);
		cursor_delay2 : in std_logic_vector(10 downto 0)
	);
end component;

component lcd_driver
	port (
		clk_main	: in std_logic;
		clk_lcd	: in std_logic;
		reset		: in std_logic;

		red		: in std_logic;
		green		: in std_logic;
		blue		: in std_logic;
		hsync		: in std_logic;
		vsync		: in std_logic;
		de			: in std_logic;

		dout0p	: out std_logic;
		dout0n	: out std_logic;
		dout1p	: out std_logic;
		dout1n	: out std_logic;
		dout2p	: out std_logic;
		dout2n	: out std_logic;
		clkoutp	: out std_logic;
		clkoutn	: out std_logic
	);
end component;

component sampler
	port (
		clk			: in std_logic;
		reset			: in std_logic;

		-- Configuration
		sampling_divider	: in std_logic_vector(4 downto 0);	-- defines number of samples in the sampling window via smp_div table

		mode			: in std_logic_vector(1 downto 0); 	-- 00: run, 01: stop, 10: single
		sampling_mode		: in std_logic_vector(1 downto 0);	-- 00: normal, 01: peak detect, 10: averaging (smoothing)

		trigger_ch0_ch1     	: in std_logic;				-- 0: ch0, 1: ch1
		trigger_rise_fall   	: in std_logic;				-- 0: rise, 1: fall
		trigger_auto_normal 	: in std_logic;				-- 0: auto, 1: normal
		trigger_level       	: in std_logic_vector(7 downto 0);	-- level in ADC data
		trigger_delay       	: in std_logic_vector(10 downto 0);	-- number of samples before trigger event
		display_roll		: in std_logic;

		-- External ADC signals
		adclk			: out std_logic;
		ad0_d			: in std_logic_vector(7 downto 0);
		ad1_d			: in std_logic_vector(7 downto 0);

		-- Sampler RAM signals
		sampler_ram_wr_addr	: out std_logic_vector(10 downto 0);
		sampler_ram_data	: out std_logic_vector(31 downto 0);
		sampler_ram_wr		: out std_logic;

		sampler_rdy		: out std_logic;
		copy_rdy		: in std_logic
	);
end component;

component controller
	port (
		clk			: in std_logic;
		reset			: in std_logic;

		display_vector		: in std_logic;
		display_roll		: in std_logic;

		-- Display RAM signals
		display_ram_wr_addr	: out std_logic_vector(10 downto 0);
		display_ram_datain	: out std_logic_vector(36 downto 0);
		display_ram_wr		: out std_logic;
		vblank			: in std_logic;

		-- Sampler RAM signals
		sampler_ram_rd_addr	: out std_logic_vector(10 downto 0);
		sampler_ram_dataout	: in std_logic_vector(31 downto 0);
		sampler_ram_wr_addr	: in std_logic_vector(10 downto 0);

		ch0_gain		: in std_logic_vector(2 downto 0);
		ch1_gain		: in std_logic_vector(2 downto 0);
		ch0_offset		: in std_logic_vector(10 downto 0);
		ch1_offset		: in std_logic_vector(10 downto 0);

		sampler_rdy		: in std_logic;
		copy_rdy		: out std_logic
	);
end component;

component ps2_mouse
	port (
		clk		: in std_logic;
		reset		: in std_logic;

		ps2_clk_in	: in std_logic;
		ps2_data_in	: in std_logic;
		ps2_clk_out	: out std_logic;
		ps2_data_out	: out std_logic;

		x		: out std_logic_vector(10 downto 0);
		y		: out std_logic_vector(9 downto 0);
		scroll		: out std_logic_vector(3 downto 0);
		buttons		: out std_logic_vector(2 downto 0)
	);
end component;

component ui
	port (
		clk		    : in std_logic;
		reset		    : in std_logic;

		mouse_x		    : in std_logic_vector(10 downto 0);
		mouse_y		    : in std_logic_vector(9 downto 0);
		mouse_buttons	    : in std_logic_vector(2 downto 0);
		mouse_scroll	    : in std_logic_vector(3 downto 0);

		mode		    : out std_logic_vector(1 downto 0);
		sampling_div	    : out std_logic_vector(4 downto 0);
		sampling_mode	    : out std_logic_vector(1 downto 0);
		sampling_realtime   : out std_logic;

		display_xt_xy	    : out std_logic;
		display_roll	    : out std_logic;
		display_vector	    : out std_logic;

		trigger_ch0_ch1     : out std_logic;
		trigger_rise_fall   : out std_logic;
		trigger_auto_normal : out std_logic;
		trigger_level       : out std_logic_vector(9 downto 0);
		trigger_delay       : out std_logic_vector(10 downto 0);

		cursor_ch1          : out std_logic;
		cursor_ch2          : out std_logic;

		cursor_en     : out std_logic;
		cursor_level1 : out std_logic_vector(9 downto 0);
		cursor_level2 : out std_logic_vector(9 downto 0);
		cursor_delay1 : out std_logic_vector(10 downto 0);
		cursor_delay2 : out std_logic_vector(10 downto 0);

		ch0_en		    : out std_logic;
		ch1_en		    : out std_logic;
		ch0_atn		    : out std_logic;
		ch1_atn		    : out std_logic;
		ch0_acdc	    : out std_logic;
		ch1_acdc	    : out std_logic;
		ch0_gain	    : out std_logic_vector(2 downto 0);
		ch1_gain	    : out std_logic_vector(2 downto 0);
		ch0_offset	    : out std_logic_vector(11 downto 0);
		ch1_offset	    : out std_logic_vector(11 downto 0);
		ch0_amp_ctrl	    : out std_logic_vector(15 downto 0);
		ch1_amp_ctrl	    : out std_logic_vector(15 downto 0);

		txt_ram_addr	    : out std_logic_vector(12 downto 0);
		txt_ram_data	    : out std_logic_vector(9 downto 0);
		txt_ram_wr	    : out std_logic;
		txt_color_ram_wr    : out std_logic
	);
end component;


end package;

package body oscilloscope_pck is
end package body;

