library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library work;
use work.oscilloscope_pck.all;

entity ps2_mouse is
	port (
		clk		: in std_logic;
		reset		: in std_logic;

		ps2_clk_in	: in std_logic;
		ps2_data_in	: in std_logic;
		ps2_clk_out	: out std_logic;
		ps2_data_out	: out std_logic;

		x		: out std_logic_vector(10 downto 0);
		y		: out std_logic_vector(9 downto 0);
		scroll		: out std_logic_vector(3 downto 0);
		buttons		: out std_logic_vector(2 downto 0)
	);
end ps2_mouse;

architecture rtl of ps2_mouse is

constant c_tx_c2d       : integer := 6000;	-- 120us @50MHz
constant c_tx_d2c       : integer := 3000;	-- 60us @50MHz
constant c_rx_timeout   : integer := 4000;	-- 80us @50MHz

signal s_ps2_clk_out_n  : std_logic;
signal s_ps2_data_out_n : std_logic;
signal s_ps2_clk_in1    : std_logic;
signal s_ps2_clk_in2    : std_logic;
signal s_ps2_data_in    : std_logic;

signal s_rxd : std_logic_vector(9 downto 0);
signal s_txd : std_logic_vector(7 downto 0);

signal s_rx_data : std_logic_vector(7 downto 0);

signal s_tx_start : std_logic;
signal s_tx_data : std_logic_vector(7 downto 0);

signal s_rx_ready : std_logic;
signal s_rx_error : std_logic;

signal s_tx_ready : std_logic;
signal s_tx_error : std_logic;

type ps2_state_type is (idle, receive, transmit);
signal ps2_state : ps2_state_type;

type mouse_state_type is (init, stream);
signal mouse_state : mouse_state_type;

signal s_signx : std_logic;
signal s_signy : std_logic;
signal s_scroll : std_logic_vector(3 downto 0);
signal s_buttons : std_logic_vector(2 downto 0);
signal s_x : std_logic_vector(10 downto 0);
signal s_y : std_logic_vector(9 downto 0);

begin

--	ps2clk  : tri port map('0', s_ps2_clk_out_n, ps2_clk);
--	ps2data : tri port map('0', s_ps2_data_out_n, ps2_data);

	ps2_clk_out <= not s_ps2_clk_out_n;
	ps2_data_out <= not s_ps2_data_out_n;

	sync : process(clk, reset)
	begin
		if reset = '1' then
			s_ps2_clk_in1 <= '0';
			s_ps2_clk_in2 <= '0';
			s_ps2_data_in <= '0';
		elsif clk'event and clk = '1' then
			s_ps2_clk_in1 <= ps2_clk_in;
			s_ps2_clk_in2 <= s_ps2_clk_in1;
			s_ps2_data_in <= ps2_data_in;
		end if;
	end process;

	ps2_comm : process(clk, reset)
	variable v_bit_cnt : integer range 0 to 15;
	variable v_parity : std_logic;
	variable v_timer : integer range 0 to 8191;
	begin
		if reset = '1' then
			ps2_state <= idle;
			v_bit_cnt := 0;
			v_parity := '0';
			v_timer := 0;
			s_rx_ready <= '0';
			s_rx_error <= '0';
			s_tx_ready <= '1';
--			s_tx_error <= '0';
			s_txd <= (others => '0');
			s_rxd <= (others => '0');
			s_rx_data <= (others => '0');
			s_ps2_clk_out_n <= '0';
			s_ps2_data_out_n <= '0';
		elsif clk'event and clk = '1' then
			case ps2_state is
			when idle =>

				if s_ps2_clk_in1 = '0' and s_ps2_clk_in2 = '1' then	-- start rcv
					if s_ps2_data_in = '0' then
						ps2_state <= receive;
						s_rx_error <= '0';
					else
						-- framing error
					end if;
				elsif s_tx_start = '1' then
					ps2_state <= transmit;
					s_txd <= s_tx_data;
					s_tx_ready <= '0';
--					s_tx_error <= '0';
				end if;
				s_rx_ready <= '0';
				v_bit_cnt := 0;
				v_parity := '0';
				v_timer := 0;

			when receive =>

				if v_timer = c_rx_timeout then	-- timeout
					s_rx_error <= '1';
					ps2_state <= idle;
				end if;
				if s_ps2_clk_in1 = s_ps2_clk_in2 then
					v_timer := v_timer + 1;
				else
					v_timer := 0;
				end if;

				if v_bit_cnt = 10 then		-- end of frame
					if s_ps2_clk_in1 = '1' and s_ps2_clk_in2 = '0' then -- rising edge
						ps2_state <= idle;
						s_rx_ready <= '1';
						s_rx_data <= s_rxd(7 downto 0);
					end if;
					if v_parity /= '0' or s_rxd(9) /= '1' then	-- parity (odd) of frame (8 bits + parity bit + stop bit) or stop bit error
						s_rx_error <= '1';
					end if;
				else
					if s_ps2_clk_in1 = '0' and s_ps2_clk_in2 = '1' then -- falling edge
						s_rxd(8 downto 0) <= s_rxd(9 downto 1);
						s_rxd(9) <= s_ps2_data_in;
						v_parity := v_parity xor s_ps2_data_in;
						v_bit_cnt := v_bit_cnt + 1;
					end if;
				end if;

			when transmit =>

				if v_bit_cnt = 0 then		-- clk_out = 0, wait for 60us
					s_ps2_clk_out_n <= '1';
					if v_timer = c_tx_c2d then 
						v_bit_cnt := v_bit_cnt + 1;
						v_timer := 0;
					else
						v_timer := v_timer + 1;
					end if;
				elsif v_bit_cnt = 1 then	-- data_out = 0, wait for 60us
					s_ps2_data_out_n <= '1';
					if v_timer = c_tx_d2c then 
						v_bit_cnt := v_bit_cnt + 1;
						v_timer := 0;
					else
						v_timer := v_timer + 1;
					end if;
				elsif v_bit_cnt = 2 then	-- clk_out = 1 and wait until clk falling edge by device
					s_ps2_clk_out_n <= '0';
					if s_ps2_clk_in1 = '0' and s_ps2_clk_in2 = '1' then
						v_bit_cnt := v_bit_cnt + 1;
					end if;
				elsif v_bit_cnt = 11 then	-- parity bit (odd parity)
					s_ps2_data_out_n <= v_parity;
					if s_ps2_clk_in1 = '0' and s_ps2_clk_in2 = '1' then
						v_bit_cnt := v_bit_cnt + 1;
					end if;
				elsif v_bit_cnt = 12 then	-- stop bit
					s_ps2_data_out_n <= '0';
					if s_ps2_clk_in1 = '0' and s_ps2_clk_in2 = '1' then
						v_bit_cnt := v_bit_cnt + 1;
					end if;
				elsif v_bit_cnt = 13 then	-- ack
--					if s_ps2_data_in = '1' then
--						s_tx_error <= '1';
--					end if;
					if s_ps2_clk_in1 = '1' and s_ps2_data_in = '1' then
						s_tx_ready <= '1';
						ps2_state <= idle;
					end if;
				else				-- wait until clk falling edge by device
					s_ps2_data_out_n <= not s_txd(0);
					if s_ps2_clk_in1 = '0' and s_ps2_clk_in2 = '1' then
						s_txd(6 downto 0) <= s_txd(7 downto 1);
						v_parity := v_parity xor s_txd(0);
						v_bit_cnt := v_bit_cnt + 1;
					end if;
				end if;

			when others =>
				ps2_state <= idle;
			end case;
		end if;
	end process;

	mouse_ctrl : process(clk, reset)
	variable v_cnt : integer range 0 to 31;
	begin
		if reset = '1' then
			mouse_state <= init;
			s_tx_data <= (others => '0');
			s_tx_start <= '0';
			v_cnt := 0;
			s_signx <= '0';
			s_signy <= '0';
			s_scroll <= (others => '0');
			s_buttons <= (others => '0');
			s_x <= conv_std_logic_vector(c_hres/2, 11);
			s_y <= conv_std_logic_vector(c_vres/2, 10);
		elsif clk'event and clk = '1' then

			case mouse_state is
			when init =>

				if v_cnt = 0 then
					s_tx_data <= x"ff";					-- reset
					s_tx_start <= '1';
					if s_tx_ready = '0' then
						s_tx_start <= '0';
						v_cnt := v_cnt + 1;
					end if;
				elsif v_cnt = 1 then
					if s_rx_ready = '1' then
						if s_rx_data = x"fa" and s_rx_error = '0' then	-- ack
							v_cnt := v_cnt + 1;
						else
							v_cnt := 0;
						end if;
					end if;
				elsif v_cnt = 2 then
					if s_rx_ready = '1' then
						if s_rx_data = x"aa" and s_rx_error = '0' then	-- self test passed
							v_cnt := v_cnt + 1;
						else
							v_cnt := 0;
						end if;
					end if;
				elsif v_cnt = 3 then
					if s_rx_ready = '1' then
						if s_rx_data = x"00" and s_rx_error = '0' then	-- mouse ID
							v_cnt := v_cnt + 1;
						else
							v_cnt := 0;
						end if;
					end if;	

				-- Init scrolling wheel
				elsif v_cnt = 4 then
					s_tx_data <= x"f3";					-- set sample rate (init scrolling wheel step 1)
					s_tx_start <= '1';
					if s_tx_ready = '0' then
						s_tx_start <= '0';
						v_cnt := v_cnt + 1;
					end if;
				elsif v_cnt = 5 then
					if s_rx_ready = '1' then
						if s_rx_data = x"fa" and s_rx_error = '0' then	-- ack
							v_cnt := v_cnt + 1;
						else
							v_cnt := 0;
						end if;
					end if;
				elsif v_cnt = 6 then
					s_tx_data <= x"c8";					-- sample rate = 200 (init scrolling wheel step 1)
					s_tx_start <= '1';
					if s_tx_ready = '0' then
						s_tx_start <= '0';
						v_cnt := v_cnt + 1;
					end if;
				elsif v_cnt = 7 then
					if s_rx_ready = '1' then
						if s_rx_data = x"fa" and s_rx_error = '0' then	-- ack
							v_cnt := v_cnt + 1;
						else
							v_cnt := 0;
						end if;
					end if;
				elsif v_cnt = 8 then
					s_tx_data <= x"f3";					-- set sample rate (init scrolling wheel step 2)
					s_tx_start <= '1';
					if s_tx_ready = '0' then
						s_tx_start <= '0';
						v_cnt := v_cnt + 1;
					end if;
				elsif v_cnt = 9 then
					if s_rx_ready = '1' then
						if s_rx_data = x"fa" and s_rx_error = '0' then	-- ack
							v_cnt := v_cnt + 1;
						else
							v_cnt := 0;
						end if;
					end if;
				elsif v_cnt = 10 then
					s_tx_data <= x"64";					-- sample rate = 100 (init scrolling wheel step 2)
					s_tx_start <= '1';
					if s_tx_ready = '0' then
						s_tx_start <= '0';
						v_cnt := v_cnt + 1;
					end if;
				elsif v_cnt = 11 then
					if s_rx_ready = '1' then
						if s_rx_data = x"fa" and s_rx_error = '0' then	-- ack
							v_cnt := v_cnt + 1;
						else
							v_cnt := 0;
						end if;
					end if;
				elsif v_cnt = 12 then
					s_tx_data <= x"f3";					-- set sample rate (init scrolling wheel step 3)
					s_tx_start <= '1';
					if s_tx_ready = '0' then
						s_tx_start <= '0';
						v_cnt := v_cnt + 1;
					end if;
				elsif v_cnt = 13 then
					if s_rx_ready = '1' then
						if s_rx_data = x"fa" and s_rx_error = '0' then	-- ack
							v_cnt := v_cnt + 1;
						else
							v_cnt := 0;
						end if;
					end if;
				elsif v_cnt = 14 then
					s_tx_data <= x"50";					-- sample rate = 80 (init scrolling wheel step 3)
					s_tx_start <= '1';
					if s_tx_ready = '0' then
						s_tx_start <= '0';
						v_cnt := v_cnt + 1;
					end if;
				elsif v_cnt = 15 then
					if s_rx_ready = '1' then
						if s_rx_data = x"fa" and s_rx_error = '0' then	-- ack
							v_cnt := v_cnt + 1;
						else
							v_cnt := 0;
						end if;
					end if;
				elsif v_cnt = 16 then
					s_tx_data <= x"f2";					-- read mouse ID
					s_tx_start <= '1';
					if s_tx_ready = '0' then
						s_tx_start <= '0';
						v_cnt := v_cnt + 1;
					end if;
				elsif v_cnt = 17 then
					if s_rx_ready = '1' then
						if s_rx_data = x"fa" and s_rx_error = '0' then	-- ack
							v_cnt := v_cnt + 1;
						else
							v_cnt := 0;
						end if;
					end if;
				elsif v_cnt = 18 then
					if s_rx_ready = '1' then
						if s_rx_data = x"03" and s_rx_error = '0' then	-- mouse ID should be 3 for scrolling mouse
							v_cnt := v_cnt + 1;
						else
							v_cnt := 0;
						end if;
					end if;	


				-- Resolution : 8 counts/mm
				elsif v_cnt = 19 then
					s_tx_data <= x"e8";					-- set resolution
					s_tx_start <= '1';
					if s_tx_ready = '0' then
						s_tx_start <= '0';
						v_cnt := v_cnt + 1;
					end if;
				elsif v_cnt = 20 then
					if s_rx_ready = '1' then
						if s_rx_data = x"fa" and s_rx_error = '0' then	-- ack
							v_cnt := v_cnt + 1;
						else
							v_cnt := 0;
						end if;
					end if;
				elsif v_cnt = 21 then
					s_tx_data <= x"03";					-- resolution : 8 counts/mm
					s_tx_start <= '1';
					if s_tx_ready = '0' then
						s_tx_start <= '0';
						v_cnt := v_cnt + 1;
					end if;
				elsif v_cnt = 22 then
					if s_rx_ready = '1' then
						if s_rx_data = x"fa" and s_rx_error = '0' then	-- ack
							v_cnt := v_cnt + 1;
						else
							v_cnt := 0;
						end if;
					end if;

				-- Enable data sending
				elsif v_cnt = 23 then
					s_tx_data <= x"f4";					-- enable data reporting
					s_tx_start <= '1';
					if s_tx_ready = '0' then
						s_tx_start <= '0';
						v_cnt := v_cnt + 1;
					end if;
				elsif v_cnt = 24 then
					if s_rx_ready = '1' then
						if s_rx_data = x"fa" and s_rx_error = '0' then	-- ack
							mouse_state <= stream;
						end if;
						v_cnt := 0;
					end if;
				end if;

			when stream =>

				if v_cnt = 0 then
					s_scroll <= "0000";
					if s_rx_ready = '1' then
						s_buttons <= s_rx_data(2 downto 0);
						s_signx <= s_rx_data(4);
						s_signy <= s_rx_data(5);
						v_cnt := v_cnt + 1;
					end if;
				elsif v_cnt = 1 then
					if s_rx_ready = '1' then
						if conv_integer(s_x + (s_signx & s_signx & s_signx & s_rx_data)) < c_hres-1 then
							s_x <= s_x + (s_signx & s_signx & s_signx & s_rx_data);
						elsif s_signx = '1' then
							s_x <= conv_std_logic_vector(0, 11);
						else
							s_x <= conv_std_logic_vector(c_hres-2, 11);
						end if;
						v_cnt := v_cnt + 1;
					end if;
				elsif v_cnt = 2 then
					if s_rx_ready = '1' then
						if conv_integer(s_y - (s_signy & s_signy & s_rx_data)) < c_vres-1 then
							s_y <= s_y - (s_signy & s_signy & s_rx_data);
						elsif s_signy = '0' then
							s_y <= conv_std_logic_vector(0, 10);
						else
							s_y <= conv_std_logic_vector(c_vres-2, 10);
						end if;
						v_cnt := v_cnt + 1;
					end if;
				elsif v_cnt = 3 then
					if s_rx_ready = '1' then
						s_scroll <= s_rx_data(3 downto 0);
						v_cnt := 0;
					end if;
				end if;

			when others =>
				mouse_state <= init;
			end case;

		end if;
	end process;

	x <= s_x;
	y <= s_y;
	scroll <= s_scroll;
	buttons <= s_buttons;

end rtl;




