library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library UNISIM;
use UNISIM.Vcomponents.ALL;

library work;
use work.oscilloscope_pck.all;

entity sampler is
	port (
		clk						: in std_logic;
		reset						: in std_logic;

		-- Configuration
		sampling_divider		: in std_logic_vector(4 downto 0);	-- defines the number of samples in the sampling window via smp_div table

		mode						: in std_logic_vector(1 downto 0); 	-- 00: run, 01: stop, 10: single
		sampling_mode			: in std_logic_vector(1 downto 0);	-- 00: normal, 01: peak detect, 10: average

		trigger_ch0_ch1		: in std_logic;							-- 0: ch0, 1: ch1
		trigger_rise_fall		: in std_logic;							-- 0: rise, 1: fall
		trigger_auto_normal	: in std_logic;							-- 0: auto, 1: normal
		trigger_level			: in std_logic_vector(7 downto 0);	-- level in ADC data
		trigger_delay			: in std_logic_vector(10 downto 0);	-- number of samples before trigger event
		display_roll			: in std_logic;							-- normal or roll display mode

		-- External ADC signals
		adclk						: out std_logic;
		ad0_d						: in std_logic_vector(7 downto 0);
		ad1_d						: in std_logic_vector(7 downto 0);

		-- Sampler RAM signals
		sampler_ram_wr_addr	: out std_logic_vector(10 downto 0);
		sampler_ram_data		: out std_logic_vector(31 downto 0);
		sampler_ram_wr			: out std_logic;

		sampler_rdy				: out std_logic;
		copy_rdy					: in std_logic
	);
end sampler;

architecture rtl of sampler is

signal s_adc0_data   : std_logic_vector(7 downto 0);
signal s_adc1_data   : std_logic_vector(7 downto 0);

signal s_ch0_sample_min : std_logic_vector(7 downto 0);
signal s_ch0_sample_max : std_logic_vector(7 downto 0);
signal s_ch1_sample_min : std_logic_vector(7 downto 0);
signal s_ch1_sample_max : std_logic_vector(7 downto 0);

signal s_sample_wr : std_logic;
signal s_sampler_ram_wr_addr : std_logic_vector(10 downto 0);

type smp_div_type is array(natural range <>) of std_logic_vector(20 downto 0);
--constant smp_div : smp_div_type(0 to 19) := (	-- for 200Msps
--  conv_std_logic_vector(      1, 21),	--   5ns/pixel, 250ns/div (50pixel/div) @200MSPS
--  conv_std_logic_vector(      2, 21),	--  10ns/pixel, 500ns/div
--  conv_std_logic_vector(      4, 21),	--  20ns/pixel,   1us/div
--  conv_std_logic_vector(      8, 21),	--  40ns/pixel,   2us/div
--  conv_std_logic_vector(     20, 21),	-- 100ns/pixel,   5us/div   
--  conv_std_logic_vector(     40, 21),	-- 200ns/pixel,  10us/div
--  conv_std_logic_vector(     80, 21),	-- 400ns/pixel,  20us/div
--  conv_std_logic_vector(    200, 21),	--   1us/pixel,  50us/div
--  conv_std_logic_vector(    400, 21),	--   2us/pixel, 100us/div
--  conv_std_logic_vector(    800, 21),	--   4us/pixel, 200us/div
--  conv_std_logic_vector(   2000, 21),	--  10us/pixel, 500us/div
--  conv_std_logic_vector(   4000, 21),	--  20us/pixel,   1ms/div
--  conv_std_logic_vector(   8000, 21),	--  40us/pixel,   2ms/div
--  conv_std_logic_vector(  20000, 21),	-- 100us/pixel,   5ms/div
--  conv_std_logic_vector(  40000, 21),	-- 200us/pixel,  10ms/div
--  conv_std_logic_vector(  80000, 21),	-- 400us/pixel,  20ms/div
--  conv_std_logic_vector( 200000, 21),	--   1ms/pixel,  50ms/div  -- 1sec/1000samples @200MSPS
--  conv_std_logic_vector( 400000, 21),	--   2ms/pixel, 100ms/div  -- 2sec/1000samples @200MSPS
--  conv_std_logic_vector( 800000, 21),	--   4ms/pixel, 200ms/div
--  conv_std_logic_vector(2000000, 21));	--  10ms/pixel, 500ms/div

constant smp_div : smp_div_type(0 to 19) := (	-- for 100Msps
  conv_std_logic_vector(      1, 21),	--  10ns/pixel, 500ns/div - 50pixel/div @100MSPS _or_ 72pixel/div @144MSPS _or_ 64pixel/div @128MSPS _or_ 80pixel/div @160MSPS _or_ 90pixel/div @180MSPS
  conv_std_logic_vector(      2, 21),	--  20ns/pixel,   1us/div
  conv_std_logic_vector(      4, 21),	--  40ns/pixel,   2us/div
  conv_std_logic_vector(     10, 21),	-- 100ns/pixel,   5us/div
  conv_std_logic_vector(     20, 21),	-- 200ns/pixel,  10us/div   
  conv_std_logic_vector(     40, 21),	-- 400ns/pixel,  20us/div
  conv_std_logic_vector(    100, 21),	--   1us/pixel,  50us/div
  conv_std_logic_vector(    200, 21),	--   2us/pixel, 100us/div
  conv_std_logic_vector(    400, 21),	--   4us/pixel, 200us/div
  conv_std_logic_vector(   1000, 21),	--  10us/pixel, 500us/div
  conv_std_logic_vector(   2000, 21),	--  20us/pixel,   1ms/div
  conv_std_logic_vector(   4000, 21),	--  40us/pixel,   2ms/div
  conv_std_logic_vector(  10000, 21),	-- 100us/pixel,   5ms/div
  conv_std_logic_vector(  20000, 21),	-- 200us/pixel,  10ms/div
  conv_std_logic_vector(  40000, 21),	-- 400us/pixel,  20ms/div
  conv_std_logic_vector( 100000, 21),	--   1ms/pixel,  50ms/div
  conv_std_logic_vector( 200000, 21),	--   2ms/pixel, 100ms/div
  conv_std_logic_vector( 400000, 21),	--   4ms/pixel, 200ms/div
  conv_std_logic_vector(1000000, 21),	--  10ms/pixel, 500ms/div
  conv_std_logic_vector(2000000, 21));	--  20ms/pixel,   1 s/div

signal s_sampling_div : std_logic_vector(20 downto 0);

signal s_trigger : std_logic;
signal s_timer_1ms : std_logic;

type state_type is (stop, copy, pretrig, trigger, sampling);
signal state : state_type;

signal s_sampling_cnt : std_logic_vector(20 downto 0);

signal s_ch0_peak_min : std_logic_vector(7 downto 0);
signal s_ch0_peak_max : std_logic_vector(7 downto 0);
signal s_ch1_peak_min : std_logic_vector(7 downto 0);
signal s_ch1_peak_max : std_logic_vector(7 downto 0);

signal s_ch0_normal : std_logic_vector(7 downto 0);
signal s_ch1_normal : std_logic_vector(7 downto 0);

signal s_ch0_avg : std_logic_vector(7 downto 0);
signal s_ch1_avg : std_logic_vector(7 downto 0);

signal s_ad0_d : std_logic_vector(7 downto 0);
signal s_ad1_d : std_logic_vector(7 downto 0);

signal s_adc0_data0 : std_logic_vector(7 downto 0);
signal s_adc1_data0 : std_logic_vector(7 downto 0);
--signal s_adc0_cnt : integer range -128 to 127;

signal s_sampling_divider		: std_logic_vector(4 downto 0);	-- defines number of samples in the sampling window via smp_div table
signal s_mode						: std_logic_vector(1 downto 0);	-- 00: run, 01: stop, 10: single
signal s_sampling_mode			: std_logic_vector(1 downto 0);	-- 00: normal, 01: peak detect, 10: average
signal s_trigger_ch0_ch1    	: std_logic;							-- 0: ch0, 1: ch1
signal s_trigger_rise_fall   	: std_logic;							-- 0: rise, 1: fall
signal s_trigger_auto_normal 	: std_logic;							-- 0: auto, 1: normal
signal s_trigger_level       	: std_logic_vector(7 downto 0);	-- level in ADC data
signal s_trigger_delay       	: std_logic_vector(10 downto 0);	-- number of samples before trigger event
signal s_display_roll			: std_logic;							-- normal or roll display mode
signal s_copy_rdy					: std_logic;							-- copy from sampler RAM to display RAM done

signal s_sampling_cnt_div : std_logic;
signal s_sc_td : std_logic;
signal s_sample_count : std_logic_vector(10 downto 0);
signal s_sampling_cnt_sync_carry : std_logic;

signal s_auto_trig_timer : std_logic_vector(7 downto 0);

signal s_sample_counter_clr : std_logic;
signal s_sample_counter_inc : std_logic;

signal s_peak_detect_rdy : std_logic;

begin

	adclk <= clk;

	clk_domain_sync : process(clk, reset)
	variable v_sampling_divider		: std_logic_vector(4 downto 0);	-- defines number of samples in the sampling window via smp_div table
	variable v_mode						: std_logic_vector(1 downto 0);	-- 00: run, 01: stop, 10: single
	variable v_sampling_mode			: std_logic_vector(1 downto 0);	-- 00: normal, 01: peak detect, 10: average
	variable v_trigger_ch0_ch1    	: std_logic;							-- 0: ch0, 1: ch1
	variable v_trigger_rise_fall   	: std_logic;							-- 0: rise, 1: fall
	variable v_trigger_auto_normal 	: std_logic;							-- 0: auto, 1: normal
	variable v_trigger_level       	: std_logic_vector(7 downto 0);	-- level in ADC data
	variable v_trigger_delay       	: std_logic_vector(10 downto 0);	-- number of samples before trigger event
	variable v_display_roll				: std_logic;							-- normal or roll display mode
	variable v_copy_rdy					: std_logic;							-- copy from sampler RAM to display RAM done
	begin
		if reset = '1' then
			s_sampling_divider		<= (others => '0');
			s_mode						<= (others => '0');
			s_sampling_mode			<= (others => '0');
			s_trigger_ch0_ch1			<= '0';
			s_trigger_rise_fall		<= '0';
			s_trigger_auto_normal	<= '0';
			s_trigger_level			<= (others => '0');
			s_trigger_delay			<= (others => '0');
			s_display_roll				<= '0';
			s_copy_rdy					<= '0';
			v_sampling_divider		:= (others => '0');
			v_mode						:= (others => '0');
			v_sampling_mode			:= (others => '0');
			v_trigger_ch0_ch1			:= '0';
			v_trigger_rise_fall		:= '0';
			v_trigger_auto_normal	:= '0';
			v_trigger_level			:= (others => '0');
			v_trigger_delay			:= (others => '0');
			v_display_roll				:= '0';
			v_copy_rdy					:= '0';
		elsif clk'event and clk = '1' then
			s_sampling_divider		<= v_sampling_divider;
			s_mode						<= v_mode;
			s_sampling_mode			<= v_sampling_mode;
			s_trigger_ch0_ch1			<= v_trigger_ch0_ch1;
			s_trigger_rise_fall		<= v_trigger_rise_fall;
			s_trigger_auto_normal	<= v_trigger_auto_normal;
			s_trigger_level			<= v_trigger_level;
			s_trigger_delay			<= v_trigger_delay;
			s_display_roll				<= v_display_roll;
			s_copy_rdy					<= v_copy_rdy;

			v_sampling_divider		:= sampling_divider;
			v_mode						:= mode;
			v_sampling_mode			:= sampling_mode;
			v_trigger_ch0_ch1			:= trigger_ch0_ch1;
			v_trigger_rise_fall		:= trigger_rise_fall;
			v_trigger_auto_normal	:= trigger_auto_normal;
			v_trigger_level			:= trigger_level;
			v_trigger_delay			:= trigger_delay;
			v_display_roll				:= display_roll;
			v_copy_rdy					:= copy_rdy;
		end if;
	end process;


--	adc_buffers: for i in 0 to 7 generate		-- the syntheser adds IBUFs, not needed manually
--		adc_data_input_buffers : IBUF 
--		generic map (
--			IBUF_DELAY_VALUE => "0",
--			IFD_DELAY_VALUE => "0"
--		)
--		port map (
--			I => ad0_d(i),
--			O => s_ad0_d(i)
--		);
--	end generate adc_buffers;
--
--	s_ad1_d <= ad1_d;


	sync : process(clk, reset)
	begin
		if reset = '1' then
			s_adc0_data0  <= (others => '0');
			s_adc1_data0  <= (others => '0');
			s_adc0_data   <= (others => '0');
			s_adc1_data   <= (others => '0');
		elsif clk'event and clk = '1' then
			s_adc0_data0  <= ad0_d;
			s_adc1_data0  <= ad1_d;
			s_adc0_data   <= s_adc0_data0;
			s_adc1_data   <= s_adc1_data0;
		end if;
	end process;

	sampling_div : process(clk, reset)
	begin
		if reset = '1' then
			s_sampling_div <= (others => '0');
		elsif clk'event and clk = '1' then
			s_sampling_div <= smp_div(conv_integer(s_sampling_divider));
		end if;
	end process;

	peak_detect_sampling: process(clk, reset)
	variable v_ch0_peak_min : std_logic_vector(7 downto 0);
	variable v_ch0_peak_max : std_logic_vector(7 downto 0);
	variable v_ch1_peak_min : std_logic_vector(7 downto 0);
	variable v_ch1_peak_max : std_logic_vector(7 downto 0);
	begin
		if reset = '1' then
			v_ch0_peak_min := x"ff";
			v_ch0_peak_max := x"00";
			v_ch1_peak_min := x"ff";
			v_ch1_peak_max := x"00";
			s_ch0_peak_min <= x"ff";
			s_ch0_peak_max <= x"00";
			s_ch1_peak_min <= x"ff";
			s_ch1_peak_max <= x"00";
		elsif clk'event and clk= '1' then
			if s_adc0_data > v_ch0_peak_max then
				v_ch0_peak_max := s_adc0_data;
			end if;
			if s_adc0_data < v_ch0_peak_min then
				v_ch0_peak_min := s_adc0_data;
			end if;
			if s_adc1_data > v_ch1_peak_max then
				v_ch1_peak_max := s_adc1_data;
			end if;
			if s_adc1_data < v_ch1_peak_min then
				v_ch1_peak_min := s_adc1_data;
			end if;
			if s_sampling_cnt_div = '1' then
				s_ch0_peak_min <= v_ch0_peak_min;
				s_ch0_peak_max <= v_ch0_peak_max;
				s_ch1_peak_min <= v_ch1_peak_min;
				s_ch1_peak_max <= v_ch1_peak_max;
				v_ch0_peak_min := x"ff";
				v_ch0_peak_max := x"00";
				v_ch1_peak_min := x"ff";
				v_ch1_peak_max := x"00";
			end if;
		end if;
	end process;

	normal_sampling: process(clk, reset)
	begin
		if reset = '1' then
			s_ch0_normal <= x"00";
			s_ch1_normal <= x"00";
		elsif clk'event and clk= '1' then
			s_ch0_normal <= s_adc0_data;
			s_ch1_normal <= s_adc1_data;
		end if;
	end process;


	average_sampling: process(clk, reset)
	variable v_ch0_avg : std_logic_vector(11 downto 0);
	variable v_ch1_avg : std_logic_vector(11 downto 0);
	begin
		if reset = '1' then
--			v_ch0_avg := x"000";
--			v_ch1_avg := x"000";
			s_ch0_avg <= x"00";
			s_ch1_avg <= x"00";
		elsif clk'event and clk= '1' then
		end if;
	end process;



	sampler_sm : process(clk, reset)
--	variable v_sample_count : std_logic_vector(10 downto 0);
	variable v_sample_count_trigger_delay : std_logic;
	variable v_sample_count_top : std_logic;
	begin
		if reset = '1' then
			state <= stop;
--			v_sample_count := (others => '0');
			sampler_rdy <= '0';
			v_sample_count_trigger_delay := '0';
			v_sample_count_top := '0';
			s_sc_td <= '0';
			s_sample_counter_clr <= '1';
			s_sample_counter_inc <= '0';
		elsif clk'event and clk = '1' then
			case state is
			when stop =>
				if s_mode = "00" then		-- run
					if s_display_roll = '1' then
						state <= sampling;
						s_sample_counter_clr <= '0';
						s_sample_counter_inc <= '0';
					else
						state <= pretrig;
						s_sample_counter_clr <= '0';
						s_sample_counter_inc <= '1';
					end if;
				end if;
--				v_sample_count := (others => '0');
				v_sample_count_trigger_delay := '0';
				v_sample_count_top := '0';
			when copy =>
				s_sample_counter_clr <= '1';
				s_sample_counter_inc <= '0';
				if s_mode = "01" then		-- stop
					state <= stop;
--					s_sample_counter_clr <= '1';
--					s_sample_counter_inc <= '0';
				elsif s_copy_rdy = '1' then
					state <= pretrig;
					sampler_rdy <= '0';
					s_sample_counter_clr <= '0';
					s_sample_counter_inc <= '1';
				end if;
--				v_sample_count := (others => '0');
				v_sample_count_trigger_delay := '0';
				v_sample_count_top := '0';
			when pretrig =>						-- collect data to fill memory before the trigger offset
				if s_mode = "01" then			-- stop
					state <= stop;
					s_sample_counter_clr <= '1';
					s_sample_counter_inc <= '0';
--				elsif v_sample_count = s_trigger_delay then		-- slow
				elsif v_sample_count_trigger_delay = '1' then
					state <= trigger;
					s_sample_counter_clr <= '0';
					s_sample_counter_inc <= '0';
				end if;
				if s_sample_count = s_trigger_delay then
					v_sample_count_trigger_delay := '1';
				end if;
--				if s_sample_wr = '1' then
--					v_sample_count := v_sample_count + 1;
--				end if;
			when trigger =>						-- continue collecting data in the circular memory and wait for the trigger signal
				if s_mode = "01" then			-- stop
					state <= stop;
					s_sample_counter_clr <= '1';
					s_sample_counter_inc <= '0';
				elsif s_trigger = '1' then
					state <= sampling;
					s_sample_counter_clr <= '0';
					s_sample_counter_inc <= '1';
				end if;
			when sampling =>						-- continue collecting data until the memory is full
				if s_mode = "01" then			-- stop
					state <= stop;
					s_sample_counter_clr <= '1';
					s_sample_counter_inc <= '0';
				elsif s_display_roll = '0' then
--					if s_sample_count = conv_std_logic_vector(1023, 10) then
					if v_sample_count_top = '1' then
						state <= copy;
						sampler_rdy <= '1';
--						s_sample_counter_clr <= '1';
--						s_sample_counter_inc <= '0';
					end if;
				else
					sampler_rdy <= '1';
				end if;
				if s_sample_count = waveform_length+2 then 	-- TBD !!!!!!!!!!
					v_sample_count_top := '1';
				else
					v_sample_count_top := '0';
				end if;
--				if s_sample_wr = '1' then
--					v_sample_count := v_sample_count + 1;
--				end if;
			when others =>
				state <= stop;
			end case;
			
		end if;
	end process;

	sample_counter : process(clk, reset)
	begin
		if reset = '1' then
			s_sample_count <= (others => '0');
		elsif clk'event and clk = '1' then
--			if state = stop or state = copy then
			if s_sample_counter_clr = '1' then
				s_sample_count <= (others => '0');
--			elsif (state = pretrig or state = sampling) and s_sample_wr = '1' then
			elsif s_sample_counter_inc = '1' and s_sample_wr = '1' then
				s_sample_count <= s_sample_count + 1;
			end if;
		end if;
	end process;
		
	sampler : process(clk, reset)
	begin
		if reset = '1' then
			s_sampling_cnt <= conv_std_logic_vector(1, 21);
			s_ch0_sample_min <= x"00";
			s_ch0_sample_max <= x"00";
			s_ch1_sample_min <= x"00";
			s_ch1_sample_max <= x"00";
			s_sample_wr <= '0';
			s_sampling_cnt_div <= '0';
			s_sampling_cnt_sync_carry <= '0';
		elsif clk'event and clk = '1' then
			s_sample_wr <= '0';
			if state = pretrig or state = trigger or state = sampling then
				if s_sampling_cnt(20 downto 0) = s_sampling_div(20 downto 0) then
--				if s_sampling_cnt_div = '1' then
					s_sample_wr <= '1';
					s_sampling_cnt <= conv_std_logic_vector(1, 21);
					s_sampling_cnt_sync_carry <= '0';
				else
--					s_sampling_cnt <= s_sampling_cnt + 1;
					s_sampling_cnt(11 downto 0) <= s_sampling_cnt(11 downto 0) + 1;
					if s_sampling_cnt(11 downto 0) = x"FFE" then
						s_sampling_cnt_sync_carry <= '1';
					else
						s_sampling_cnt_sync_carry <= '0';
					end if;
					if s_sampling_cnt_sync_carry = '1' then
						s_sampling_cnt(20 downto 12) <= s_sampling_cnt(20 downto 12) + 1;
					end if;
				end if;

				if s_sampling_cnt_div = '1' then
					if s_sampling_mode = "00" then			-- normal mode: store every Nth sample (aliasing!!! - should be modified)
						s_ch0_sample_min <= s_ch0_normal;
						s_ch0_sample_max <= s_ch0_normal;
						s_ch1_sample_min <= s_ch1_normal;
						s_ch1_sample_max <= s_ch1_normal;
					elsif s_sampling_mode = "01" then		-- peak detect: store the min and max out of N samples
						s_ch0_sample_min <= s_ch0_peak_min;
						s_ch0_sample_max <= s_ch0_peak_max;
						s_ch1_sample_min <= s_ch1_peak_min;
						s_ch1_sample_max <= s_ch1_peak_max;
					elsif s_sampling_mode = "10" then		-- averaging (not implemented)
						s_ch0_sample_min <= s_ch0_avg;
						s_ch0_sample_max <= s_ch0_avg;
						s_ch1_sample_min <= s_ch1_avg;
						s_ch1_sample_max <= s_ch1_avg;
					end if;
				end if;
			end if;

			if s_sampling_cnt(20 downto 0) = s_sampling_div(20 downto 0) then
				s_sampling_cnt_div <= '1';
			else
				s_sampling_cnt_div <= '0';
			end if;
		end if;
	end process;


	
	trigger_signal : process(clk, reset)
	variable v_trigger : std_logic;
	variable v_trigger_state : std_logic;
	variable v_trigger_level_n : std_logic_vector(7 downto 0);
	variable v_trigger_level_p : std_logic_vector(7 downto 0);
	variable v_cmp0 : std_logic_vector(1 downto 0);
	variable v_cmp1 : std_logic_vector(1 downto 0);
	variable v_auto_trig_timer_top : std_logic;
	begin
		if reset = '1' then
			v_trigger := '0';
			s_auto_trig_timer <= (others => '0');
			s_trigger <= '0';
			v_trigger_state := '0';
			v_trigger_level_n := (others => '0');
			v_trigger_level_p := (others => '0');
			v_auto_trig_timer_top := '0';
		elsif clk'event and clk = '1' then

			if s_trigger_auto_normal = '0' then		-- auto
				if v_trigger = '1' or v_auto_trig_timer_top = '1' then
					s_trigger <= '1';
				else
					s_trigger <= '0';
				end if;
				if v_trigger = '1' or v_auto_trig_timer_top = '1' then
					s_auto_trig_timer <= (others => '0');
				elsif s_timer_1ms = '1' then
					s_auto_trig_timer <= s_auto_trig_timer + 1;
				end if;
			else									-- normal
				s_trigger <= v_trigger;
				s_auto_trig_timer <= (others => '0');
			end if;

			if s_auto_trig_timer = 200 then			-- 100ms
				v_auto_trig_timer_top := '1';
			else
				v_auto_trig_timer_top := '0';
			end if;
			
			v_trigger := '0';

			if v_trigger_state = '0' then
				if s_trigger_ch0_ch1 = '0' and s_trigger_rise_fall = '0' and v_cmp0(0) = '1' then	--s_adc0_data < v_trigger_level_n then
					v_trigger_state := '1';
				elsif s_trigger_ch0_ch1 = '0' and s_trigger_rise_fall = '1' and v_cmp0(1) = '1' then 	--s_adc0_data > v_trigger_level_p then
					v_trigger_state := '1';
				elsif s_trigger_ch0_ch1 = '1' and s_trigger_rise_fall = '0' and v_cmp1(0) = '1'	then	--s_adc1_data < v_trigger_level_n then
					v_trigger_state := '1';
				elsif s_trigger_ch0_ch1 = '1' and s_trigger_rise_fall = '1' and v_cmp1(1) = '1' then	--s_adc1_data > v_trigger_level_p then
					v_trigger_state := '1';
				end if;
			else 
				if s_trigger_ch0_ch1 = '0' and s_trigger_rise_fall = '0' and v_cmp0(1) = '1' then 	--s_adc0_data > v_trigger_level_p then
					v_trigger_state := '0';
					v_trigger := '1';
				elsif s_trigger_ch0_ch1 = '0' and s_trigger_rise_fall = '1' and v_cmp0(0) = '1' then	--s_adc0_data < v_trigger_level_n then
					v_trigger_state := '0';
					v_trigger := '1';
				elsif s_trigger_ch0_ch1 = '1' and s_trigger_rise_fall = '0' and v_cmp1(1) = '1' then	--s_adc1_data > v_trigger_level_p then
					v_trigger_state := '0';
					v_trigger := '1';
				elsif s_trigger_ch0_ch1 = '1' and s_trigger_rise_fall = '1' and v_cmp1(0) = '1'	then	--s_adc1_data < v_trigger_level_n then
					v_trigger_state := '0';
					v_trigger := '1';
				end if;
			end if;

			if s_adc0_data < v_trigger_level_n then
				v_cmp0(0) := '1';
			else
				v_cmp0(0) := '0';
			end if;

			if s_adc0_data > v_trigger_level_p then
				v_cmp0(1) := '1';
			else
				v_cmp0(1) := '0';
			end if;

			if s_adc1_data < v_trigger_level_n then
				v_cmp1(0) := '1';
			else
				v_cmp1(0) := '0';
			end if;

			if s_adc1_data > v_trigger_level_p then
				v_cmp1(1) := '1';
			else
				v_cmp1(1) := '0';
			end if;

			v_trigger_level_n := s_trigger_level-1;
			v_trigger_level_p := s_trigger_level+1;
		end if;
	end process;

	timer_1ms : process(clk, reset)
	variable v_timer : std_logic_vector(14 downto 0);
	begin
		if reset = '1' then
			s_timer_1ms <= '1';
			v_timer := (others => '0');
		elsif clk'event and clk = '1' then
			if v_timer = 24999 then						-- 0.5ms
				v_timer := (others => '0');
				s_timer_1ms <= '1';
			else
				v_timer := v_timer + 1;
				s_timer_1ms <= '0';
			end if;
		end if;
	end process;

	sampler_ram_driver : process(clk, reset)
	begin
		if reset = '1' then
			s_sampler_ram_wr_addr <= (others => '0');
			sampler_ram_data <= (others => '0');
			sampler_ram_wr <= '0';
		elsif clk'event and clk = '1' then
			-- roll mode
			if s_sample_wr = '1' then
				s_sampler_ram_wr_addr <= s_sampler_ram_wr_addr + 1;
			end if;
			sampler_ram_wr <= s_sample_wr;
			sampler_ram_data <= s_ch1_sample_max & s_ch1_sample_min & s_ch0_sample_max & s_ch0_sample_min;
		end if;
	end process;

	sampler_ram_wr_addr <= s_sampler_ram_wr_addr;

end rtl;

