-- Oscilloscope for Xilinx Spartan-3E, made by Peter Szabo
-- https://opencores.org/projects/xilinx_scope

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library work;
use work.oscilloscope_pck.all;
use work.xilinx_pck.all;

Library UNISIM;
use UNISIM.vcomponents.all;
 
entity sys is
port ( 
	clk : in std_logic;
	rst : in std_logic;

	vga_red   : out std_logic;				-- VGA does not work since the display_driver module timings are changed for FlatLink output
	vga_green : out std_logic;
	vga_blue  : out std_logic;
	vga_hsync : out std_logic;
	vga_vsync : out std_logic;

	lcd_dout0p	: out std_logic;			-- FlatLink
	lcd_dout0n	: out std_logic;
	lcd_dout1p	: out std_logic;
	lcd_dout1n	: out std_logic;
	lcd_dout2p	: out std_logic;
	lcd_dout2n	: out std_logic;
	lcd_clkoutp	: out std_logic;
	lcd_clkoutn	: out std_logic;

	ps2_clk   : inout std_logic;			-- mouse
	ps2_data  : inout std_logic;
	
	adclk		: out std_logic;
	ad0_d		: in std_logic_vector(7 downto 0);
--	ad1_d		: in std_logic_vector(7 downto 0);

	generator_1kHz	: out std_logic;
	pll_locked 		: out std_logic
);
end sys;

architecture rtl of sys is

signal rst1, rst2, s_reset : std_logic;
signal s_clk_main				: std_logic;
signal s_clk_lcd				: std_logic;
signal s_clk_sampler			: std_logic;
signal s_pll_locked			: std_logic;

signal s_display_ram_rd_addr		: std_logic_vector(10 downto 0);
signal s_display_ram_dataout		: std_logic_vector(36 downto 0);
signal s_display_ram_wr_addr		: std_logic_vector(10 downto 0);
signal s_display_ram_wr 			: std_logic;
signal s_display_ram_wr_vector 	: std_logic_vector(0 downto 0);
signal s_display_ram_datain		: std_logic_vector(36 downto 0);

signal s_sampler_ram_rd_addr		: std_logic_vector(10 downto 0);
signal s_sampler_ram_dataout		: std_logic_vector(31 downto 0);
signal s_sampler_ram_wr_addr		: std_logic_vector(10 downto 0);
signal s_sampler_ram_wr 			: std_logic;
signal s_sampler_ram_wr_vector 	: std_logic_vector(0 downto 0);
signal s_sampler_ram_datain		: std_logic_vector(31 downto 0);
signal s_sampling_div				: std_logic_vector(4 downto 0);
signal s_sampling_mode				: std_logic_vector(1 downto 0);
signal s_mode							: std_logic_vector(1 downto 0);

signal s_txt_ram_rd_addr			: std_logic_vector(12 downto 0);
signal s_txt_ram_dataout			: std_logic_vector(9 downto 0);
signal s_txt_ram_wr_addr			: std_logic_vector(12 downto 0);
signal s_txt_ram_wr 					: std_logic;
signal s_txt_color_ram_wr			: std_logic;
signal s_txt_ram_wr_vector 		: std_logic_vector(0 downto 0);
signal s_txt_color_ram_wr_vector	: std_logic_vector(0 downto 0);
signal s_txt_ram_datain				: std_logic_vector(9 downto 0);

signal s_ch0_en			: std_logic;
signal s_ch1_en			: std_logic;
signal s_ch0_gain			: std_logic_vector(2 downto 0);
signal s_ch1_gain			: std_logic_vector(2 downto 0);
signal s_ch0_offset		: std_logic_vector(11 downto 0);
signal s_ch1_offset		: std_logic_vector(11 downto 0);
signal s_ch0_amp_ctrl	: std_logic_vector(15 downto 0);
signal s_ch1_amp_ctrl	: std_logic_vector(15 downto 0);

signal s_vblank : std_logic;

signal s_mouse_x			: std_logic_vector(10 downto 0);
signal s_mouse_y			: std_logic_vector(9 downto 0);
signal s_mouse_scroll	: std_logic_vector(3 downto 0);
signal s_mouse_buttons	: std_logic_vector(2 downto 0);

signal s_ps2_clk_in		: std_logic;
signal s_ps2_data_in		: std_logic;
signal s_ps2_clk_out		: std_logic;
signal s_ps2_data_out	: std_logic;

signal s_trigger_ch0_ch1		: std_logic;
signal s_trigger_rise_fall		: std_logic;
signal s_trigger_auto_normal	: std_logic;
signal s_trigger_level			: std_logic_vector(9 downto 0);
signal s_trigger_delay			: std_logic_vector(10 downto 0);

signal s_display_roll	: std_logic;
signal s_sampler_rdy		: std_logic;
signal s_copy_rdy			: std_logic;

signal s_display_vector : std_logic;

--signal ad0_d : std_logic_vector(7 downto 0);
signal ad1_d	: std_logic_vector(7 downto 0);

signal s_red	: std_logic;
signal s_green	: std_logic;
signal s_blue	: std_logic;
signal s_hsync	: std_logic;
signal s_vsync	: std_logic;
signal s_de		: std_logic;

signal s_cursor_en		: std_logic;
signal s_cursor_level1	: std_logic_vector(9 downto 0);
signal s_cursor_delay1	: std_logic_vector(10 downto 0);
signal s_cursor_level2	: std_logic_vector(9 downto 0);
signal s_cursor_delay2	: std_logic_vector(10 downto 0);

begin

	dcm1: dcm
	port map (
		clk			=> clk,
		reset			=> rst,
		clk_main		=> s_clk_main,
		clk_lcd		=> s_clk_lcd,
		clk_sampler	=> s_clk_sampler,
		clk_adc		=> adclk,
		locked		=> s_pll_locked
--		status0		=> ,						-- status register should be verified for DCM status ('locked' bits are not enough)
--		status1		=> ,
--		status2		=> ,
--		status3		=> 
	);

	s_reset <= not s_pll_locked;

	pll_locked <= s_pll_locked;		-- LED

----------------------------------------------------------
-- Display driver
----------------------------------------------------------

	display_driver1 : display_driver
	port map (
		clk 			=> s_clk_main,
		reset 		=> s_reset,

		wave_addr	=> s_display_ram_rd_addr,
		wave_data	=> s_display_ram_dataout,

		txt_addr		=> s_txt_ram_rd_addr,
		txt_data		=> s_txt_ram_dataout,

		vblank		=> s_vblank,
		ch0_en		=> s_ch0_en,
		ch1_en		=> s_ch1_en,
		ch0_offset	=> s_ch0_offset(10 downto 0),
		ch1_offset	=> s_ch1_offset(10 downto 0),

		red 			=> s_red,
		green 		=> s_green,
		blue 			=> s_blue,
		hsync 		=> s_hsync,
		vsync 		=> s_vsync,
		de				=> s_de,

		mouse_x		=> s_mouse_x,
		mouse_y		=> s_mouse_y,

		trigger_level	=> s_trigger_level,
		trigger_delay	=> s_trigger_delay,
				
		cursor_en     => s_cursor_en,
		cursor_level1 => s_cursor_level1,
		cursor_level2 => s_cursor_level2,
		cursor_delay1 => s_cursor_delay1,
		cursor_delay2 => s_cursor_delay2
	);

	display_ram1 : display_ram
	port map (
		clka 	=> s_clk_main,
		dina 	=> s_display_ram_datain,
		addra => s_display_ram_wr_addr,
		wea 	=> s_display_ram_wr_vector,
		clkb 	=> s_clk_main,
		addrb => s_display_ram_rd_addr,
		doutb => s_display_ram_dataout
	);

	s_display_ram_wr_vector(0) <= s_display_ram_wr;

	vga_red		<= s_red;
	vga_green	<= s_green;
	vga_blue		<= s_blue;
	vga_hsync	<= s_hsync;
	vga_vsync	<= s_vsync;


----------------------------------------------------------
-- LCD driver
----------------------------------------------------------

	lcd_driver1: lcd_driver
	port map (
		clk_main	=> s_clk_main,
		clk_lcd	=> s_clk_lcd,
		reset		=> s_reset,

		red		=> s_red,
		green		=> s_green,
		blue		=> s_blue,
		hsync		=> s_hsync,
		vsync		=> s_vsync,
		de			=> s_de,

		dout0p	=> lcd_dout0p,
		dout0n	=> lcd_dout0n,
		dout1p	=> lcd_dout1p,
		dout1n	=> lcd_dout1n,
		dout2p	=> lcd_dout2p,
		dout2n	=> lcd_dout2n,
		clkoutp	=> lcd_clkoutp,
		clkoutn	=> lcd_clkoutn
	);


----------------------------------------------------------
-- Sampler
----------------------------------------------------------

	sampler1 : sampler
	port map (
		clk			=> s_clk_sampler,
		reset			=> s_reset,

		-- Configuration
		sampling_divider	=> s_sampling_div,

		mode			=> s_mode,
		sampling_mode		=> s_sampling_mode,

		trigger_ch0_ch1     	=> s_trigger_ch0_ch1,
		trigger_rise_fall   	=> s_trigger_rise_fall,
		trigger_auto_normal 	=> s_trigger_auto_normal,
		trigger_level       	=> s_trigger_level(9 downto 2),
		trigger_delay       	=> s_trigger_delay,
		display_roll		=> s_display_roll,

		-- External ADC signals
--		adclk			=> adclk,
		ad0_d			=> ad0_d,
		ad1_d			=> ad1_d,

		sampler_ram_wr_addr	=> s_sampler_ram_wr_addr,
		sampler_ram_data	=> s_sampler_ram_datain,
		sampler_ram_wr		=> s_sampler_ram_wr,

		sampler_rdy		=> s_sampler_rdy,
		copy_rdy		=> s_copy_rdy
	);

	sampler_ram1 : sampler_ram
	port map (
		clka 	=> s_clk_sampler,
		dina 	=> s_sampler_ram_datain,
		addra => s_sampler_ram_wr_addr,
		wea 	=> s_sampler_ram_wr_vector,
		clkb 	=> s_clk_main,
		addrb => s_sampler_ram_rd_addr,
		doutb => s_sampler_ram_dataout
	);

	s_sampler_ram_wr_vector(0) <= s_sampler_ram_wr;



--ad1_d <= x"80";

	process(s_clk_main, s_reset)
	begin
		if s_reset = '1' then
			ad1_d <= x"00";
		elsif s_clk_main'event and s_clk_main = '1' then
			ad1_d <= ad1_d + 1;
		end if;
	end process;


----------------------------------------------------------
-- Controller
----------------------------------------------------------

	controller1 : controller
	port map (
		clk				=> s_clk_main,
		reset			=> s_reset,

		display_vector		=> s_display_vector,
		display_roll		=> s_display_roll,

		-- Display RAM signals
		display_ram_wr_addr	=> s_display_ram_wr_addr,
		display_ram_datain	=> s_display_ram_datain,
		display_ram_wr		=> s_display_ram_wr,
		vblank			=> s_vblank,

		-- Sampler RAM signals
		sampler_ram_rd_addr	=> s_sampler_ram_rd_addr,
		sampler_ram_dataout	=> s_sampler_ram_dataout,
		sampler_ram_wr_addr	=> s_sampler_ram_wr_addr,

		ch0_gain		=> s_ch0_gain,
		ch1_gain		=> s_ch1_gain,
		ch0_offset		=> s_ch0_offset(10 downto 0),
		ch1_offset		=> s_ch1_offset(10 downto 0),

		sampler_rdy		=> s_sampler_rdy,
		copy_rdy		=> s_copy_rdy
	);


----------------------------------------------------------
-- PS2 mouse
----------------------------------------------------------

	ps2_mouse1 : ps2_mouse
	port map (
		clk		=> s_clk_main,
		reset		=> s_reset,

		ps2_clk_in	 => s_ps2_clk_in,
		ps2_data_in	 => s_ps2_data_in,
		ps2_clk_out	 => s_ps2_clk_out,
		ps2_data_out => s_ps2_data_out,

		x		=> s_mouse_x,
		y		=> s_mouse_y,
		scroll		=> s_mouse_scroll,
		buttons		=> s_mouse_buttons
	);

	s_ps2_clk_in  <= ps2_clk;
	s_ps2_data_in <= ps2_data;

	ps2_clk  <= 'Z' when s_ps2_clk_out = '1' else '0';
	ps2_data <= 'Z' when s_ps2_data_out = '1' else '0';


----------------------------------------------------------
-- User interface
----------------------------------------------------------

	ui1 : ui
	port map (
		clk		 => s_clk_main,
		reset		 => s_reset,

		mouse_x		 => s_mouse_x,
		mouse_y		 => s_mouse_y,
		mouse_buttons	 => s_mouse_buttons,
		mouse_scroll	 => s_mouse_scroll,

		mode		 => s_mode,
		sampling_div	 => s_sampling_div,
		sampling_mode	 => s_sampling_mode,
--		sampling_realtime   : out std_logic;

--		display_xt_xy	    : out std_logic;
		display_roll	 => s_display_roll,
		display_vector	 => s_display_vector,

		trigger_ch0_ch1     => s_trigger_ch0_ch1,
		trigger_rise_fall   => s_trigger_rise_fall,
		trigger_auto_normal => s_trigger_auto_normal,
		trigger_level	     => s_trigger_level,
		trigger_delay	     => s_trigger_delay,

--		cursor_ch1          : out std_logic;
--		cursor_ch2          : out std_logic;

		cursor_en     => s_cursor_en,
		cursor_level1 => s_cursor_level1,
		cursor_level2 => s_cursor_level2,
		cursor_delay1 => s_cursor_delay1,
		cursor_delay2 => s_cursor_delay2,

		ch0_en		=> s_ch0_en,
		ch1_en		=> s_ch1_en,
--		ch0_atn		=> ch0_atn,						-- 1:1 or 1:10 division at the analog input (should control an external relay/switch)
--		ch1_atn		=> ch1_atn,
--		ch0_acdc		=> ch0_acdc,					-- AC or DC coupling (external)
--		ch1_acdc		=> ch1_acdc,
		ch0_gain		=> s_ch0_gain,
		ch1_gain		=> s_ch1_gain,
		ch0_offset	=> s_ch0_offset,
		ch1_offset	=> s_ch1_offset,
--		ch0_amp_ctrl	=> s_ch0_amp_ctrl,
--		ch1_amp_ctrl	=> s_ch1_amp_ctrl,

		txt_ram_addr	 => s_txt_ram_wr_addr,
		txt_ram_data	 => s_txt_ram_datain,
		txt_ram_wr		 => s_txt_ram_wr,
		txt_color_ram_wr => s_txt_color_ram_wr
	);

	txt_ram1 : txt_ram
	port map (
		clka 	=> s_clk_main,
		dina 	=> s_txt_ram_datain(6 downto 0),
		addra => s_txt_ram_wr_addr,
		wea	=> s_txt_ram_wr_vector,
		clkb 	=> s_clk_main,
		addrb => s_txt_ram_rd_addr,
		doutb	=> s_txt_ram_dataout(6 downto 0)
	);

	txt_color_ram1: txt_color_ram
	port map (
		clka  => s_clk_main,
		dina  => s_txt_ram_datain(9 downto 7),
		addra => s_txt_ram_wr_addr,
		wea   => s_txt_color_ram_wr_vector,
		clkb  => s_clk_main,
		addrb => s_txt_ram_rd_addr,
		doutb => s_txt_ram_dataout(9 downto 7)
	);

	s_txt_ram_wr_vector(0) <= s_txt_ram_wr;
	s_txt_color_ram_wr_vector(0) <= s_txt_color_ram_wr;


----------------------------------------------------------
-- 1kHz generator
----------------------------------------------------------

	process(s_clk_main, s_reset)
	variable v_cnt : integer range 0 to 32767;
	variable v_generator_1kHz : std_logic;
	begin
		if s_reset = '1' then
			v_cnt := 0;
			v_generator_1kHz := '0';
		elsif s_clk_main'event and s_clk_main = '1' then	-- 40 or 45MHz (setting in dcm.vhd)
--			if v_cnt = 19999 then									-- 40MHz/1kHz/2 = 20000
			if v_cnt = 22499 then									-- 45MHz/1kHz/2 = 22500
				v_cnt := 0;
				v_generator_1kHz := not v_generator_1kHz;
			else
				v_cnt := v_cnt + 1;
			end if;
		end if;
		generator_1kHz <= v_generator_1kHz;
	end process;

end rtl;
