library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library work;
use work.oscilloscope_pck.all;

entity ui is
	port (
		clk		    : in std_logic;
		reset		    : in std_logic;

		mouse_x		    : in std_logic_vector(10 downto 0);
		mouse_y		    : in std_logic_vector(9 downto 0);
		mouse_buttons	 : in std_logic_vector(2 downto 0);
		mouse_scroll	 : in std_logic_vector(3 downto 0);

		mode					: out std_logic_vector(1 downto 0);
		sampling_div	   : out std_logic_vector(4 downto 0);
		sampling_mode	   : out std_logic_vector(1 downto 0);
		sampling_realtime	: out std_logic;

		display_xt_xy	    : out std_logic;
		display_roll	    : out std_logic;
		display_vector	    : out std_logic;

		trigger_ch0_ch1     : out std_logic;
		trigger_rise_fall   : out std_logic;
		trigger_auto_normal : out std_logic;
		trigger_level       : out std_logic_vector(9 downto 0);
		trigger_delay       : out std_logic_vector(10 downto 0);

		cursor_ch1          : out std_logic;
		cursor_ch2          : out std_logic;

		cursor_en     : out std_logic;
		cursor_level1 : out std_logic_vector(9 downto 0);
		cursor_level2 : out std_logic_vector(9 downto 0);
		cursor_delay1 : out std_logic_vector(10 downto 0);
		cursor_delay2 : out std_logic_vector(10 downto 0);

		ch0_en		 : out std_logic;
		ch1_en		 : out std_logic;
		ch0_atn		 : out std_logic;
		ch1_atn		 : out std_logic;
		ch0_acdc	    : out std_logic;
		ch1_acdc	    : out std_logic;
		ch0_gain	    : out std_logic_vector(2 downto 0);
		ch1_gain	    : out std_logic_vector(2 downto 0);
		ch0_offset	 : out std_logic_vector(11 downto 0);
		ch1_offset	 : out std_logic_vector(11 downto 0);
		ch0_amp_ctrl : out std_logic_vector(15 downto 0);
		ch1_amp_ctrl : out std_logic_vector(15 downto 0);

		txt_ram_addr	    : out std_logic_vector(12 downto 0);
		txt_ram_data	    : out std_logic_vector(9 downto 0);
		txt_ram_wr	       : out std_logic;
		txt_color_ram_wr   : out std_logic
	);
end ui;

architecture rtl of ui is

type txt_ram_state_type is (init, wr);
signal txt_ram_state : txt_ram_state_type;

signal s_txt_ram_wr  : std_logic;
signal s_txt_addr    : std_logic_vector(11 downto 0);
signal s_txt_idx     : integer range 0 to 255;
signal s_txt_count   : integer range 0 to 31;
--signal s_txt_ram_rdy : std_logic;

signal s_mouse_buttons         : std_logic_vector(2 downto 0);
signal s_mouse_buttons_press   : std_logic_vector(2 downto 0);
signal s_mouse_buttons_release : std_logic_vector(2 downto 0);
signal s_mouse_scroll_event    : std_logic;
signal s_mouse_scroll          : std_logic_vector(11 downto 0);

constant timebase : string :=
"500 n" &
"  1 ~" &
"  2 ~" &
"  5 ~" &
" 10 ~" &
" 20 ~" &
" 50 ~" &
"100 ~" &
"200 ~" &
"500 ~" &
"  1 m" &
"  2 m" &
"  5 m" &
" 10 m" &
" 20 m" &
" 50 m" &
"100 m" &
"200 m" &
"500 m" &
"  1  " ;

constant vperdiv : string :=
"  2  " &
"  1  " &
"500 m" &
"250 m" &
"125 m";

type button_record_type is record 
	x      : integer range 0 to 127;	-- x start position on display (in character counts)
	y      : integer range 0 to 63;	-- y start position on display (in character counts)
	dx     : integer range 0 to 15; 	-- x length on display (in character counts)
end record;

constant c_button_count : integer := 40;

type t_buttons is array (0 to c_button_count-1) of button_record_type;
constant c_button : t_buttons :=
(( 0,  0, 10),		--  0 xxx ms/div
 (38, 32,  7),		--  1 Trigger -xxx ms
 (91,  0,  3),		--  2 RUN
 (94,  0,  1),		--  3 /
 (95,  0,  4),		--  4 STOP
 (100, 0,  6),		--  5 SINGLE
 ( 2, 30,  2),		--  6 CH1 on/off
 ( 6, 30,  2),		--  7 CH1 dc
 (10, 30,  2),		--  8 CH1 ac
 ( 2, 31, 10),		--  9 CH1 xxx mV/div
 ( 4, 32,  6),		-- 10 CH1 offset 1
 ( 4, 33,  6),		-- 11 CH1 offset 2	- not used
 (15, 30,  2),		-- 12 CH2 on/off
 (19, 30,  2),		-- 13 CH2 dc
 (23, 30,  2),		-- 14 CH2 ac
 (15, 31, 10),		-- 15 CH2 xxx mV/div
 (17, 32,  6),		-- 16 CH2 offset 1
 (17, 33,  6),		-- 17 CH2 offset 2	- not used
 (36, 29,  3),		-- 18 Trigger ch1
 (42, 29,  3),		-- 19 Trigger ch2
 (30, 30,  4),		-- 20 Trigger rise
 (30, 31,  4),		-- 21 Trigger fall
 (39, 30,  4),		-- 22 Trigger auto
 (39, 31,  6),		-- 23 Trigger normal
 (29, 32,  7),		-- 24 Trigger -xxx mV
 (49, 30,  6),		-- 25 Acquire normal
 (49, 31, 11),		-- 26 Acquire peak detect
 (49, 32,  7),		-- 27 Acquire average
 (57, 32,  3),		-- 28 Acquire 1..256
 (57, 30,  3),		-- 29 Acquire @realtime
 (63, 30,  3),		-- 30 Display X-t
 (63, 31,  3),		-- 31 Display X-Y
 (68, 30,  4),		-- 32 Display roll
 (63, 32,  6),		-- 33 Display vector
 (87, 29,  3),		-- 34 Cursor ch1
 (93, 29,  3),		-- 35 Cursor ch2
 (76, 30, 10),		-- 36 Cursor V1 -xxx mV
 (76, 31, 10),		-- 37 Cursor V2 -xxx mV
 (88, 30, 10),		-- 38 Cursor t1 -xxx ms
 (88, 31, 10));	-- 39 Cursor t2 -xxx ms

type button_pos_record_type is record 
	x0     : integer range 0 to 2047;	-- x start position on display (pixels)
	y0     : integer range 0 to 1023;	-- y start position on display (pixels)
	x1     : integer range 0 to 2047; 	-- x end position on display (pixels)
	y1     : integer range 0 to 1023; 	-- y end position on display (pixels)
end record;

type t_buttons_pos is array (0 to c_button_count-1) of button_pos_record_type;
constant c_button_pos : t_buttons_pos :=
((0,                   0,                   12*(c_button(0).x+c_button(0).dx)+4,   24*(c_button(0).y+1)+4),		--  0 xxx ms/div
 (12*c_button(1).x+4,  24*c_button(1).y+4,  12*(c_button(1).x+c_button(1).dx)+4,   c_vres),							--  1 Trigger -xxx ms
 (12*c_button(2).x+4,  0,                   12*(c_button(2).x+c_button(2).dx)+4,   24*(c_button(2).y+1)+4),		--  2 RUN
 (12*c_button(3).x+4,  0,                   12*(c_button(3).x+c_button(3).dx)+4,   24*(c_button(3).y+1)+4),		--  3 /
 (12*c_button(4).x+4,  0,                   12*(c_button(4).x+c_button(4).dx)+4,   24*(c_button(4).y+1)+4),		--  4 STOP
 (12*c_button(5).x+4,  0,                   c_hres,                                24*(c_button(5).y+1)+4),		--  5 SINGLE
 (12*c_button(6).x+4,  24*c_button(6).y+4,  12*(c_button(6).x+c_button(6).dx)+4,   24*(c_button(6).y+1)+4),
 (12*c_button(7).x+4,  24*c_button(7).y+4,  12*(c_button(7).x+c_button(7).dx)+4,   24*(c_button(7).y+1)+4),
 (12*c_button(8).x+4,  24*c_button(8).y+4,  12*(c_button(8).x+c_button(8).dx)+4,   24*(c_button(8).y+1)+4),
 (12*c_button(9).x+4,  24*c_button(9).y+4,  12*(c_button(9).x+c_button(9).dx)+4,   24*(c_button(9).y+1)+4),
 (12*c_button(10).x+4, 24*c_button(10).y+4, 12*(c_button(10).x+c_button(10).dx)+4, c_vres),							-- 10 CH1 offset 1
 (12*c_button(11).x+4, 24*c_button(11).y+4, 12*(c_button(11).x+c_button(11).dx)+4, 24*(c_button(11).y+1)+4),
 (12*c_button(12).x+4, 24*c_button(12).y+4, 12*(c_button(12).x+c_button(12).dx)+4, 24*(c_button(12).y+1)+4),
 (12*c_button(13).x+4, 24*c_button(13).y+4, 12*(c_button(13).x+c_button(13).dx)+4, 24*(c_button(13).y+1)+4),
 (12*c_button(14).x+4, 24*c_button(14).y+4, 12*(c_button(14).x+c_button(14).dx)+4, 24*(c_button(14).y+1)+4),
 (12*c_button(15).x+4, 24*c_button(15).y+4, 12*(c_button(15).x+c_button(15).dx)+4, 24*(c_button(15).y+1)+4),
 (12*c_button(16).x+4, 24*c_button(16).y+4, 12*(c_button(16).x+c_button(16).dx)+4, c_vres),							-- 16 CH2 offset 1
 (12*c_button(17).x+4, 24*c_button(17).y+4, 12*(c_button(17).x+c_button(17).dx)+4, 24*(c_button(17).y+1)+4),
 (12*c_button(18).x+4, 24*c_button(18).y+4, 12*(c_button(18).x+c_button(18).dx)+4, 24*(c_button(18).y+1)+4),
 (12*c_button(19).x+4, 24*c_button(19).y+4, 12*(c_button(19).x+c_button(19).dx)+4, 24*(c_button(19).y+1)+4),
 (12*c_button(20).x+4, 24*c_button(20).y+4, 12*(c_button(20).x+c_button(20).dx)+4, 24*(c_button(20).y+1)+4),
 (12*c_button(21).x+4, 24*c_button(21).y+4, 12*(c_button(21).x+c_button(21).dx)+4, 24*(c_button(21).y+1)+4),
 (12*c_button(22).x+4, 24*c_button(22).y+4, 12*(c_button(22).x+c_button(22).dx)+4, 24*(c_button(22).y+1)+4),
 (12*c_button(23).x+4, 24*c_button(23).y+4, 12*(c_button(23).x+c_button(23).dx)+4, 24*(c_button(23).y+1)+4),
 (12*c_button(24).x+4, 24*c_button(24).y+4, 12*(c_button(24).x+c_button(24).dx)+4, c_vres),							-- 24 Trigger -xxx mV
 (12*c_button(25).x+4, 24*c_button(25).y+4, 12*(c_button(25).x+c_button(25).dx)+4, 24*(c_button(25).y+1)+4),
 (12*c_button(26).x+4, 24*c_button(26).y+4, 12*(c_button(26).x+c_button(26).dx)+4, 24*(c_button(26).y+1)+4),
 (12*c_button(27).x+4, 24*c_button(27).y+4, 12*(c_button(27).x+c_button(27).dx)+4, c_vres),							-- 27 Acquire average
 (12*c_button(28).x+4, 24*c_button(28).y+4, 12*(c_button(28).x+c_button(28).dx)+4, c_vres),							-- 28 Acquire 1..256
 (12*c_button(29).x+4, 24*c_button(29).y+4, 12*(c_button(29).x+c_button(29).dx)+4, 24*(c_button(29).y+1)+4),
 (12*c_button(30).x+4, 24*c_button(30).y+4, 12*(c_button(30).x+c_button(30).dx)+4, 24*(c_button(30).y+1)+4),
 (12*c_button(31).x+4, 24*c_button(31).y+4, 12*(c_button(31).x+c_button(31).dx)+4, 24*(c_button(31).y+1)+4),
 (12*c_button(32).x+4, 24*c_button(32).y+4, 12*(c_button(32).x+c_button(32).dx)+4, 24*(c_button(32).y+1)+4),
 (12*c_button(33).x+4, 24*c_button(33).y+4, 12*(c_button(33).x+c_button(33).dx)+4, c_vres),							-- 33 Display vector
 (12*c_button(34).x+4, 24*c_button(34).y+4, 12*(c_button(34).x+c_button(34).dx)+4, 24*(c_button(34).y+1)+4),
 (12*c_button(35).x+4, 24*c_button(35).y+4, 12*(c_button(35).x+c_button(35).dx)+4, 24*(c_button(35).y+1)+4),
 (12*c_button(36).x+4, 24*c_button(36).y+4, 12*(c_button(36).x+c_button(36).dx)+4, 24*(c_button(36).y+1)+4),
 (12*c_button(37).x+4, 24*c_button(37).y+4, 12*(c_button(37).x+c_button(37).dx)+4, 24*(c_button(37).y+1)+4),
 (12*c_button(38).x+4, 24*c_button(38).y+4, 12*(c_button(38).x+c_button(38).dx)+4, 24*(c_button(38).y+1)+4),
 (12*c_button(39).x+4, 24*c_button(39).y+4, 12*(c_button(39).x+c_button(39).dx)+4, 24*(c_button(39).y+1)+4));

type t_button_addr is array (0 to c_button_count-1) of integer range 0 to 3999;	-- address of buttons in the text RAM
constant c_button_addr : t_button_addr := 
(c_button(0).y*106+c_button(0).x,
 c_button(1).y*106+c_button(1).x,
 c_button(2).y*106+c_button(2).x,
 c_button(3).y*106+c_button(3).x,
 c_button(4).y*106+c_button(4).x,
 c_button(5).y*106+c_button(5).x,
 c_button(6).y*106+c_button(6).x,
 c_button(7).y*106+c_button(7).x,
 c_button(8).y*106+c_button(8).x,
 c_button(9).y*106+c_button(9).x,
 c_button(10).y*106+c_button(10).x,
 c_button(11).y*106+c_button(11).x,
 c_button(12).y*106+c_button(12).x,
 c_button(13).y*106+c_button(13).x,
 c_button(14).y*106+c_button(14).x,
 c_button(15).y*106+c_button(15).x,
 c_button(16).y*106+c_button(16).x,
 c_button(17).y*106+c_button(17).x,
 c_button(18).y*106+c_button(18).x,
 c_button(19).y*106+c_button(19).x,
 c_button(20).y*106+c_button(20).x,
 c_button(21).y*106+c_button(21).x,
 c_button(22).y*106+c_button(22).x,
 c_button(23).y*106+c_button(23).x,
 c_button(24).y*106+c_button(24).x,
 c_button(25).y*106+c_button(25).x,
 c_button(26).y*106+c_button(26).x,
 c_button(27).y*106+c_button(27).x,
 c_button(28).y*106+c_button(29).x,
 c_button(29).y*106+c_button(29).x,
 c_button(30).y*106+c_button(30).x,
 c_button(31).y*106+c_button(31).x,
 c_button(32).y*106+c_button(32).x,
 c_button(33).y*106+c_button(33).x,
 c_button(34).y*106+c_button(34).x,
 c_button(35).y*106+c_button(35).x,
 c_button(36).y*106+c_button(36).x,
 c_button(37).y*106+c_button(37).x,
 c_button(38).y*106+c_button(38).x,
 c_button(39).y*106+c_button(39).x);



signal s_target : std_logic_vector(c_button_count-1 downto 0);


type event_type is array (natural range <>) of std_logic_vector(1 downto 0);
signal s_event : event_type(c_button_count-1 downto 0);

type button_color_type is array (natural range <>) of std_logic_vector(2 downto 0);
signal s_color : button_color_type(c_button_count-1 downto 0);

signal s_mode : std_logic_vector(1 downto 0);

signal s_ch1_on     : std_logic;
signal s_ch2_on     : std_logic;
signal s_ch2_acdc   : std_logic;
signal s_ch1_acdc   : std_logic;
signal s_ch1_offset : std_logic_vector(11 downto 0);
signal s_ch2_offset : std_logic_vector(11 downto 0);

signal s_sampling_div      : std_logic_vector(4 downto 0);
signal s_sampling_mode     : std_logic_vector(1 downto 0);
signal s_sampling_realtime : std_logic;

signal s_display_xt_xy  : std_logic;
signal s_display_roll   : std_logic;
signal s_display_vector : std_logic;

signal s_trigger_ch1_ch2     : std_logic;
signal s_trigger_rise_fall   : std_logic;
signal s_trigger_auto_normal : std_logic;
signal s_trigger_level       : std_logic_vector(9 downto 0);
signal s_trigger_delay       : std_logic_vector(10 downto 0);

signal s_cursor_ch1 : std_logic;
signal s_cursor_ch2 : std_logic;

signal s_cursor_ch1_level1 : std_logic_vector(9 downto 0);
signal s_cursor_ch1_delay1 : std_logic_vector(10 downto 0);
signal s_cursor_ch1_level2 : std_logic_vector(9 downto 0);
signal s_cursor_ch1_delay2 : std_logic_vector(10 downto 0);

signal s_cursor_ch2_level1 : std_logic_vector(9 downto 0);
signal s_cursor_ch2_delay1 : std_logic_vector(10 downto 0);
signal s_cursor_ch2_level2 : std_logic_vector(9 downto 0);
signal s_cursor_ch2_delay2 : std_logic_vector(10 downto 0);

signal s_timer_1ms : std_logic;

type gain_type is array(0 to 32) of std_logic_vector(5 downto 0);
constant c_gain : gain_type :=
('0' & '0' & x"A",	-- -20dB +10dB -20dB
 '0' & '0' & x"9",	-- -20dB +10dB -18dB
 '0' & '0' & x"8",	-- -20dB +10dB -16dB
 '0' & '0' & x"7",	-- -20dB +10dB -14dB
 '0' & '0' & x"6",	-- -20dB +10dB -12dB
 '0' & '0' & x"5",	-- -20dB +10dB -10dB
 '0' & '0' & x"4",	-- -20dB +10dB - 8dB
 '0' & '0' & x"3",	-- -20dB +10dB - 6dB
 '0' & '0' & x"2",	-- -20dB +10dB - 4dB
 '0' & '0' & x"1",	-- -20dB +10dB - 2dB
 '0' & '0' & x"0",	-- -20dB +10dB - 0dB	-- remove
 '1' & '0' & x"A",	-- - 0dB +10dB -20dB
 '1' & '0' & x"9",	-- - 0dB +10dB -18dB
 '1' & '0' & x"8",	-- - 0dB +10dB -16dB
 '1' & '0' & x"7",	-- - 0dB +10dB -14dB
 '1' & '0' & x"6",	-- - 0dB +10dB -12dB
 '1' & '0' & x"5",	-- - 0dB +10dB -10dB
 '1' & '0' & x"4",	-- - 0dB +10dB - 8dB
 '1' & '0' & x"3",	-- - 0dB +10dB - 6dB
 '1' & '0' & x"2",	-- - 0dB +10dB - 4dB
 '1' & '0' & x"1",	-- - 0dB +10dB - 2dB
 '1' & '0' & x"0",	-- - 0dB +10dB - 0dB	
 '1' & '1' & x"A",	-- - 0dB +30dB -20dB	-- remove
 '1' & '1' & x"9",	-- - 0dB +30dB -18dB
 '1' & '1' & x"8",	-- - 0dB +30dB -16dB
 '1' & '1' & x"7",	-- - 0dB +30dB -14dB
 '1' & '1' & x"6",	-- - 0dB +30dB -12dB
 '1' & '1' & x"5",	-- - 0dB +30dB -10dB
 '1' & '1' & x"4",	-- - 0dB +30dB - 8dB
 '1' & '1' & x"3",	-- - 0dB +30dB - 6dB
 '1' & '1' & x"2",	-- - 0dB +30dB - 4dB
 '1' & '1' & x"1",	-- - 0dB +30dB - 2dB
 '1' & '1' & x"0");	-- - 0dB +30dB - 0dB

--signal s_ch1_gain : integer range 0 to 32;
--signal s_ch2_gain : integer range 0 to 32;
signal s_ch1_gain : std_logic_vector(5 downto 0);
signal s_ch2_gain : std_logic_vector(5 downto 0);

begin

-------------------------------------------------------------------------
-- mouse events
-------------------------------------------------------------------------

	mouse_buttons_press_release : process(clk, reset)
	begin
		if reset = '1' then
			s_mouse_buttons <= (others => '0');
			s_mouse_buttons_press <= (others => '0');
			s_mouse_buttons_release <= (others => '0');
		elsif clk'event and clk = '1' then
			s_mouse_buttons <= mouse_buttons;
			s_mouse_buttons_press <= mouse_buttons and not s_mouse_buttons;
			s_mouse_buttons_release <= not mouse_buttons and s_mouse_buttons;
		end if;
	end process;

	mouse_scroll_fwd_rev : process(clk, reset)
	variable v_scroll_timeout : integer range 0 to 10000000;	-- 200 ms
	begin
		if reset = '1' then
			v_scroll_timeout := 0;
			s_mouse_scroll_event <= '0';
			s_mouse_scroll(3 downto 0) <= x"0";
		elsif clk'event and clk = '1' then
			if v_scroll_timeout = 0 then	-- limit scrolling speed to one count / 200ms
				if mouse_scroll /= x"0" then
					s_mouse_scroll_event <= '1';
					s_mouse_scroll(3 downto 0) <= mouse_scroll;
					-- v_scroll_timeout := 10000000;
					v_scroll_timeout := 1;
				end if;
			else
				s_mouse_scroll_event <= '0';
				v_scroll_timeout := v_scroll_timeout - 1;
			end if;
		end if;
	end process;

	scroll_sign : for i in 4 to 11 generate
		s_mouse_scroll(i) <= s_mouse_scroll(3);
	end generate;

	mouse_target : process(clk, reset)
	begin
		if reset = '1' then
			s_target <= (others => '0');
		elsif clk'event and clk = '1' then
			s_target <= (others => '0');
			for i in 0 to c_button_count-1 loop
				if mouse_x >= c_button_pos(i).x0 and mouse_x < c_button_pos(i).x1 and mouse_y >= c_button_pos(i).y0 and mouse_y < c_button_pos(i).y1 then
						s_target(i) <= '1';
				end if;
			end loop;
		end if;	
	end process;	

	events: for i in 0 to c_button_count-1 generate
		s_event(i)(0) <= s_mouse_buttons_press(0) and s_target(i);
		s_event(i)(1) <= s_mouse_scroll_event and s_target(i);
	end generate;

-------------------------------------------------------------------------
-- Mode select : run/stop, single
-------------------------------------------------------------------------

	mode_select : process(clk, reset)
	begin
		if reset = '1' then
			s_mode <= (others => '0');
		elsif clk'event and clk = '1' then
			if s_event(2)(0) = '1' or s_event(3)(0) = '1' or s_event(4)(0) = '1' then
				if s_mode = "00" then
					s_mode <= "01";		-- stop
				elsif s_mode = "01" or s_mode = "10" then
					s_mode <= "00";		-- run
				end if;
			elsif s_event(5)(0) = '1' then
				s_mode <= "10";			-- single
			end if;
		end if;
	end process;

	mode <= s_mode;
	s_color(2) <= "010" when s_mode = "00" else "111";
	s_color(3) <= "111";
	s_color(4) <= "100" when s_mode = "01" else "111";
	s_color(5) <= "010" when s_mode = "10" else "111";

-------------------------------------------------------------------------
-- CH 1 settings
-------------------------------------------------------------------------

	ch1_settings : process(clk, reset)
	begin
		if reset = '1' then
			s_ch1_on <= '1';
			s_ch1_acdc <= '0';
			s_ch1_gain <= conv_std_logic_vector(2, 6);
		elsif clk'event and clk = '1' then
			if s_event(6)(0) = '1' then
				s_ch1_on <= not s_ch1_on;
			end if;
			if s_event(7)(0) = '1' then
				s_ch1_acdc <= '0';
			elsif s_event(8)(0) = '1' then
				s_ch1_acdc <= '1';
			end if;
			if s_event(9)(1) = '1' then
				if s_mouse_scroll(3) = '1' and s_ch1_gain < 4 then 
					s_ch1_gain <= s_ch1_gain + 1;
				elsif s_mouse_scroll(3) = '0' and s_ch1_gain > 0 then 
					s_ch1_gain <= s_ch1_gain - 1;
				end if;
			end if;
		end if;
	end process;

	ch1_offset_drag : process(clk, reset)
	variable v_adjust : std_logic;
	begin
		if reset = '1' then
			s_ch1_offset <= x"800";
			v_adjust := '0';
		elsif clk'event and clk = '1' then
--			if s_event(10)(1) = '1' or s_event(11)(1) = '1' then
			if s_event(10)(1) = '1' then
				s_ch1_offset <= s_ch1_offset - s_mouse_scroll;
			end if;

			if s_mouse_buttons_release(0) = '1' then
				v_adjust := '0';
			elsif s_mouse_buttons_press(0) = '1' then
				if conv_integer(mouse_x) >= x0-40 and
				   conv_integer(mouse_x) <= x0-25 and
				   conv_integer(mouse_y) >= y0+ny/2*dy+2048-s_ch1_offset-5 and
				   conv_integer(mouse_y) <= y0+ny/2*dy+2048-s_ch1_offset+15 then
					v_adjust := '1';
				end if;
			end if;

			if v_adjust = '1' then
				if conv_integer(mouse_y) < y0 then 
					s_ch1_offset <= conv_std_logic_vector(2048 + ny/2*dy, 12);
				elsif conv_integer(mouse_y) > y0+ny*dy then
					s_ch1_offset <= conv_std_logic_vector(2048 - ny/2*dy, 12);
				else
					s_ch1_offset <= conv_std_logic_vector(y0+ny/2*dy+2048 - conv_integer(mouse_y), 12);
				end if;
			end if;
		end if;
	end process;

	ch0_en <= s_ch1_on;
	s_color(6) <= "010" when s_ch1_on = '1' else "111";
	ch0_acdc <= s_ch1_acdc;
	s_color(7) <= "010" when s_ch1_acdc = '0' else "111";
	s_color(8) <= "010" when s_ch1_acdc = '1' else "111";
	ch0_atn <= c_gain(conv_integer(s_ch1_gain))(5);
	ch0_amp_ctrl(4 downto 0) <= c_gain(conv_integer(s_ch1_gain))(4 downto 0);
	ch0_amp_ctrl(15 downto 5) <= "000" & x"00";
	ch0_gain <= s_ch1_gain(2 downto 0);
	ch0_offset <= not s_ch1_offset(11) & s_ch1_offset(10 downto 0);
	s_color(9)  <= "010" when s_target(9) = '1' else "111";
--	s_color(10) <= "010" when s_target(10) = '1' or s_target(11) = '1' else "111";
--	s_color(11) <= "010" when s_target(10) = '1' or s_target(11) = '1' else "111";
	s_color(10) <= "010" when s_target(10) = '1' else "111";
	s_color(11) <= "010" when s_target(10) = '1' else "111";

-------------------------------------------------------------------------
-- CH 2 settings
-------------------------------------------------------------------------

	ch2_settings : process(clk, reset)
	begin
		if reset = '1' then
			s_ch2_on <= '1';
			s_ch2_acdc <= '0';
			s_ch2_gain <= conv_std_logic_vector(2, 6);
		elsif clk'event and clk = '1' then
			if s_event(12)(0) = '1' then
				s_ch2_on <= not s_ch2_on;
			end if;
			if s_event(13)(0) = '1' then
				s_ch2_acdc <= '0';
			elsif s_event(14)(0) = '1' then
				s_ch2_acdc <= '1';
			end if;
			if s_event(15)(1) = '1' then
				if s_mouse_scroll(3) = '1' and s_ch2_gain < 4 then 
					s_ch2_gain <= s_ch2_gain + 1;
				elsif s_mouse_scroll(3) = '0' and s_ch2_gain > 0 then 
					s_ch2_gain <= s_ch2_gain - 1;
				end if;
			end if;
		end if;
	end process;

	ch2_offset_drag : process(clk, reset)
	variable v_adjust : std_logic;
	begin
		if reset = '1' then
			s_ch2_offset <= x"800";
			v_adjust := '0';
		elsif clk'event and clk = '1' then
--			if s_event(16)(1) = '1' or s_event(17)(1) = '1' then
			if s_event(16)(1) = '1' then
				s_ch2_offset <= s_ch2_offset - s_mouse_scroll;
			end if;

			if s_mouse_buttons_release(0) = '1' then
				v_adjust := '0';
			elsif s_mouse_buttons_press(0) = '1' then
				if conv_integer(mouse_x) >= x0-20 and
				   conv_integer(mouse_x) <= x0-5 and
				   conv_integer(mouse_y) >= y0+ny/2*dy+2048-s_ch2_offset-5 and
				   conv_integer(mouse_y) <= y0+ny/2*dy+2048-s_ch2_offset+15 then
					v_adjust := '1';
				end if;
			end if;

			if v_adjust = '1' then
				if conv_integer(mouse_y) < y0 then 
					s_ch2_offset <= conv_std_logic_vector(2048 + ny/2*dy, 12);
				elsif conv_integer(mouse_y) > y0+ny*dy then
					s_ch2_offset <= conv_std_logic_vector(2048 - ny/2*dy, 12);
				else
					s_ch2_offset <= conv_std_logic_vector(y0+ny/2*dy+2048 - conv_integer(mouse_y), 12);
				end if;
			end if;
		end if;
	end process;

	ch1_en <= s_ch2_on;
	s_color(12) <= "100" when s_ch2_on = '1' else "111";
	ch1_acdc <= s_ch2_acdc;
	s_color(13) <= "100" when s_ch2_acdc = '0' else "111";
	s_color(14) <= "100" when s_ch2_acdc = '1' else "111";
	ch1_atn <= c_gain(conv_integer(s_ch2_gain))(5);
	ch1_amp_ctrl(4 downto 0) <= c_gain(conv_integer(s_ch2_gain))(4 downto 0);
	ch1_amp_ctrl(15 downto 5) <= "000" & x"00";
	ch1_gain <= s_ch2_gain(2 downto 0);
	ch1_offset <= not s_ch2_offset(11) & s_ch2_offset(10 downto 0);
	s_color(15) <= "100" when s_target(15) = '1' else "111";
--	s_color(16) <= "100" when s_target(16) = '1' or s_target(17) = '1' else "111";
--	s_color(17) <= "100" when s_target(16) = '1' or s_target(17) = '1' else "111";
	s_color(16) <= "100" when s_target(16) = '1' else "111";
	s_color(17) <= "100" when s_target(16) = '1' else "111";

-------------------------------------------------------------------------
-- Trigger settings
-------------------------------------------------------------------------

	trigger_settings : process(clk, reset)
--	variable v_timeout : integer range 0 to 2047;
	begin
		if reset = '1' then
			s_trigger_ch1_ch2 <= '0';
			s_trigger_rise_fall <= '0';
			s_trigger_auto_normal <= '0';
		elsif clk'event and clk = '1' then
			if s_event(18)(0) = '1' then
				s_trigger_ch1_ch2 <= '0';
			elsif s_event(19)(0) = '1' then
				s_trigger_ch1_ch2 <= '1';
			end if;
			if s_event(20)(0) = '1' then
				s_trigger_rise_fall <= '0';
			elsif s_event(21)(0) = '1' then
				s_trigger_rise_fall <= '1';
			end if;
			if s_event(22)(0) = '1' then
				s_trigger_auto_normal <= '0';
			elsif s_event(23)(0) = '1' then
				s_trigger_auto_normal <= '1';
			end if;
		end if;
	end process;

	trigger_drag : process(clk, reset)
	variable v_adjust_level : std_logic;
	variable v_adjust_delay : std_logic;
	begin
		if reset = '1' then
			s_trigger_level <= conv_std_logic_vector(512, 10);
			s_trigger_delay <= conv_std_logic_vector(waveform_length/2, 11);
			v_adjust_level := '0';
			v_adjust_delay := '0';
		elsif clk'event and clk = '1' then

			if s_mouse_buttons_release(0) = '1' then
				v_adjust_level := '0';
				v_adjust_delay := '0';
			elsif s_mouse_buttons_press(0) = '1' then
				if conv_integer(mouse_x) >= x0 and conv_integer(mouse_x) <= x0+nx*dx and
				   conv_integer(mouse_y) > y0+ny/2*dy+512 - conv_integer(s_trigger_level) - 10 and
				   conv_integer(mouse_y) < y0+ny/2*dy+512 - conv_integer(s_trigger_level) + 10 then
					v_adjust_level := '1';
				end if;
				if conv_integer(mouse_y) >= y0 and conv_integer(mouse_y) <= y0+ny*dy and
				      conv_integer(mouse_x) > x0 + conv_integer(s_trigger_delay) - 10 and
				      conv_integer(mouse_x) < x0 + conv_integer(s_trigger_delay) + 10 then
					v_adjust_delay := '1';
				end if;
			end if;

			if v_adjust_level = '1' then
				if conv_integer(mouse_y) < y0 then 
					s_trigger_level <= conv_std_logic_vector(512 + ny/2*dy, 10);
				elsif conv_integer(mouse_y) > y0+ny*dy then
					s_trigger_level <= conv_std_logic_vector(512 - ny/2*dy, 10);
				else
					s_trigger_level <= conv_std_logic_vector(y0+ny/2*dy+512 - conv_integer(mouse_y), 10);
				end if;
			else
				if s_event(24)(1) = '1' then								-- trigger level button, mouse scroll event
					s_trigger_level <= s_trigger_level - s_mouse_scroll(9 downto 0);
				end if;
			end if;

			if v_adjust_delay = '1' then
				if conv_integer(mouse_x) < x0 then 
					s_trigger_delay <= conv_std_logic_vector(0, 11);
				elsif conv_integer(mouse_x) > x0+nx*dx then
					s_trigger_delay <= conv_std_logic_vector(waveform_length, 11);
				else
					s_trigger_delay <= conv_std_logic_vector(conv_integer(mouse_x) - x0, 11);
				end if;
			else
				if s_event(1)(1) = '1' then								-- trigger delay button, mouse scroll event
					s_trigger_delay <= s_trigger_delay - s_mouse_scroll(10 downto 0);
				end if;
			end if;

		end if;
	end process;

	trigger_ch0_ch1 <= s_trigger_ch1_ch2;
	trigger_rise_fall <= s_trigger_rise_fall;
	trigger_auto_normal <= s_trigger_auto_normal;
	trigger_level <= s_trigger_level;
	trigger_delay <= s_trigger_delay;
	s_color(18) <= "010" when s_trigger_ch1_ch2 = '0' else "111";
	s_color(19) <= "100" when s_trigger_ch1_ch2 = '1' else "111";

	s_color(20) <= "010" when s_trigger_rise_fall = '0' and s_trigger_ch1_ch2 = '0' else
						"100" when s_trigger_rise_fall = '0' and s_trigger_ch1_ch2 = '1' else
						"111";

	s_color(21) <= "010" when s_trigger_rise_fall = '1' and s_trigger_ch1_ch2 = '0' else
						"100" when s_trigger_rise_fall = '1' and s_trigger_ch1_ch2 = '1' else
						"111";

	s_color(22) <= "010" when s_trigger_auto_normal = '0' and s_trigger_ch1_ch2 = '0' else
						"100" when s_trigger_auto_normal = '0' and s_trigger_ch1_ch2 = '1' else
						"111";

	s_color(23) <= "010" when s_trigger_auto_normal = '1' and s_trigger_ch1_ch2 = '0' else
						"100" when s_trigger_auto_normal = '1' and s_trigger_ch1_ch2 = '1' else
						"111";

	s_color(24) <= "010" when s_target(24) = '1' and s_trigger_ch1_ch2 = '0' else
						"100" when s_target(24) = '1' and s_trigger_ch1_ch2 = '1' else
						"111";

	s_color(1)  <= "010" when s_target(1) = '1' and s_trigger_ch1_ch2 = '0' else
						"100" when s_target(1) = '1' and s_trigger_ch1_ch2 = '1' else
						"111";

-------------------------------------------------------------------------
-- Acquire mode : normal, peak detect, averaging, realtime
-------------------------------------------------------------------------

	sampling_acquire_mode_select : process(clk, reset)
	begin
		if reset = '1' then
			s_sampling_mode <= "01";		-- peak detect by default
			s_sampling_realtime <= '0';
		elsif clk'event and clk = '1' then
			if s_event(25)(0) = '1' then
				s_sampling_mode <= "00";	-- normal
			elsif s_event(26)(0) = '1' then
				s_sampling_mode <= "01";	-- peak detect
			elsif s_event(27)(0) = '1' then
				s_sampling_mode <= "10";	-- average
			end if;
			if s_event(29)(0) = '1' then
				s_sampling_realtime <= not s_sampling_realtime;	-- realtime
			end if;
		end if;
	end process;

	sampling_mode <= s_sampling_mode;
	sampling_realtime <= s_sampling_realtime;
	s_color(25) <= "010" when s_sampling_mode = "00" else "111";
	s_color(26) <= "010" when s_sampling_mode = "01" else "111";
	s_color(27) <= "010" when s_sampling_mode = "10" else "111";
	s_color(28) <= "010" when s_target(28) = '1' else "111";
	s_color(29) <= "010" when s_sampling_realtime = '1' else "111";

-------------------------------------------------------------------------
-- Timebase
-------------------------------------------------------------------------

	sampling_divide : process(clk, reset)
	begin
		if reset = '1' then
			s_sampling_div <= conv_std_logic_vector(12, 5);	-- 5ms/div @200Msps
--			s_sampling_div <= conv_std_logic_vector(0, 5);
		elsif clk'event and clk = '1' then
			-- 20 different sampling rates 
			if s_event(0)(1) = '1' and s_sampling_div < conv_std_logic_vector(19, 5) and s_mouse_scroll(3) = '0' then
				s_sampling_div <= s_sampling_div + 1;
			elsif s_event(0)(1) = '1' and s_sampling_div > conv_std_logic_vector(0, 5) and s_mouse_scroll(3) = '1' then
				s_sampling_div <= s_sampling_div - 1;
			end if;
		end if;
	end process;

	sampling_div <= s_sampling_div;
	s_color(0)  <= "010" when s_target(0) = '1' else "111";

-------------------------------------------------------------------------
-- Display settings : X-t, X-Y, roll mode, vector display
-------------------------------------------------------------------------

	display_settings : process(clk, reset)
	begin
		if reset = '1' then
			s_display_xt_xy <= '0';
			s_display_roll <= '0';
			s_display_vector <= '1';
		elsif clk'event and clk = '1' then
			if s_event(30)(0) = '1' then
				s_display_xt_xy <= '0';
			elsif s_event(31)(0) = '1' then
				s_display_xt_xy <= '1';
				s_display_roll <= '0';
			elsif s_event(32)(0) = '1' and s_display_roll = '0' then
				s_display_roll <= '1';
				s_display_xt_xy <= '0';
			elsif s_event(32)(0) = '1' and s_display_roll = '1' then
				s_display_roll <= '0';
			end if;
			if s_event(33)(0) = '1' then
				s_display_vector <= not s_display_vector;
			end if;
		end if;
	end process;

	display_xt_xy <= s_display_xt_xy;
	display_roll <= s_display_roll;
	display_vector <= s_display_vector;
	s_color(30) <= "010" when s_display_xt_xy  = '0' else "111";
	s_color(31) <= "010" when s_display_xt_xy  = '1' else "111";
	s_color(32) <= "010" when s_display_roll   = '1' else "111";
	s_color(33) <= "010" when s_display_vector = '1' else "111";

-------------------------------------------------------------------------
-- Cursor settings
-------------------------------------------------------------------------

	cursor_settings : process(clk, reset)
	variable v_adjust_ch1_level1 : std_logic;
	variable v_adjust_ch1_level2 : std_logic;
	variable v_adjust_ch1_delay1 : std_logic;
	variable v_adjust_ch1_delay2 : std_logic;
	variable v_adjust_ch2_level1 : std_logic;
	variable v_adjust_ch2_level2 : std_logic;
	variable v_adjust_ch2_delay1 : std_logic;
	variable v_adjust_ch2_delay2 : std_logic;
	begin
		if reset = '1' then
			s_cursor_ch1 <= '0';
			s_cursor_ch2 <= '0';

			s_cursor_ch1_level1 <= conv_std_logic_vector(300, 10);
			s_cursor_ch1_level2 <= conv_std_logic_vector(600, 10);
			s_cursor_ch1_delay1 <= conv_std_logic_vector(700, 11);
			s_cursor_ch1_delay2 <= conv_std_logic_vector(900, 11);

			s_cursor_ch2_level1 <= conv_std_logic_vector(400, 10);
			s_cursor_ch2_level2 <= conv_std_logic_vector(700, 10);
			s_cursor_ch2_delay1 <= conv_std_logic_vector(800, 11);
			s_cursor_ch2_delay2 <= conv_std_logic_vector(1000, 11);

			v_adjust_ch1_level1 := '0';
			v_adjust_ch1_level2 := '0';
			v_adjust_ch1_delay1 := '0';
			v_adjust_ch1_delay2 := '0';
			v_adjust_ch2_level1 := '0';
			v_adjust_ch2_level2 := '0';
			v_adjust_ch2_delay1 := '0';
			v_adjust_ch2_delay2 := '0';
		elsif clk'event and clk = '1' then

			if s_event(34)(0) = '1' and s_cursor_ch1 = '0' then
				s_cursor_ch1 <= '1';
				s_cursor_ch2 <= '0';
			elsif s_event(34)(0) = '1' and s_cursor_ch1 = '1' then
				s_cursor_ch1 <= '0';
				s_cursor_ch2 <= '0';
			elsif s_event(35)(0) = '1' and s_cursor_ch2 = '0' then
				s_cursor_ch1 <= '0';
				s_cursor_ch2 <= '1';
			elsif s_event(35)(0) = '1' and s_cursor_ch2 = '1' then
				s_cursor_ch1 <= '0';
				s_cursor_ch2 <= '0';
			end if;

			if s_mouse_buttons_release(0) = '1' then
				v_adjust_ch1_level1 := '0';
				v_adjust_ch1_level2 := '0';
				v_adjust_ch1_delay1 := '0';
				v_adjust_ch1_delay2 := '0';
				v_adjust_ch2_level1 := '0';
				v_adjust_ch2_level2 := '0';
				v_adjust_ch2_delay1 := '0';
				v_adjust_ch2_delay2 := '0';
			elsif s_mouse_buttons_press(0) = '1' then
				if s_cursor_ch1 = '1' then
					if conv_integer(mouse_x) > x0+nx*dx and conv_integer(mouse_x) <= x0+nx*dx+20 and
						conv_integer(mouse_y) > y0+ny/2*dy+512 - conv_integer(s_cursor_ch1_level1) - 10 and
						conv_integer(mouse_y) < y0+ny/2*dy+512 - conv_integer(s_cursor_ch1_level1) + 10 then
						v_adjust_ch1_level1 := '1';
					elsif conv_integer(mouse_x) > x0+nx*dx and conv_integer(mouse_x) <= x0+nx*dx+20 and
						conv_integer(mouse_y) > y0+ny/2*dy+512 - conv_integer(s_cursor_ch1_level2) - 10 and
						conv_integer(mouse_y) < y0+ny/2*dy+512 - conv_integer(s_cursor_ch1_level2) + 10 then
						v_adjust_ch1_level2 := '1';
					end if;
					if conv_integer(mouse_y) > y0+ny*dy and conv_integer(mouse_y) <= y0+ny*dy+20 and
						conv_integer(mouse_x) > x0+nx/2*dx-1024 + conv_integer(s_cursor_ch1_delay1) - 10 and
						conv_integer(mouse_x) < x0+nx/2*dx-1024 + conv_integer(s_cursor_ch1_delay1) + 10 then
						v_adjust_ch1_delay1 := '1';
					elsif conv_integer(mouse_y) > y0+ny*dy and conv_integer(mouse_y) <= y0+ny*dy+20 and
						conv_integer(mouse_x) > x0+nx/2*dx-1024 + conv_integer(s_cursor_ch1_delay2) - 10 and
						conv_integer(mouse_x) < x0+nx/2*dx-1024 + conv_integer(s_cursor_ch1_delay2) + 10 then
						v_adjust_ch1_delay2 := '1';
					end if;
				end if;
				if s_cursor_ch2 = '1' then
					if conv_integer(mouse_x) > x0+nx*dx and conv_integer(mouse_x) <= x0+nx*dx+20 and
						conv_integer(mouse_y) > y0+ny/2*dy+512 - conv_integer(s_cursor_ch2_level1) - 10 and
						conv_integer(mouse_y) < y0+ny/2*dy+512 - conv_integer(s_cursor_ch2_level1) + 10 then
						v_adjust_ch2_level1 := '1';
					elsif conv_integer(mouse_x) > x0+nx*dx and conv_integer(mouse_x) <= x0+nx*dx+20 and
						conv_integer(mouse_y) > y0+ny/2*dy+512 - conv_integer(s_cursor_ch2_level2) - 10 and
						conv_integer(mouse_y) < y0+ny/2*dy+512 - conv_integer(s_cursor_ch2_level2) + 10 then
						v_adjust_ch2_level2 := '1';
					end if;
					if conv_integer(mouse_y) > y0+ny*dy and conv_integer(mouse_y) <= y0+ny*dy+20 and
						conv_integer(mouse_x) > x0+nx/2*dx-1024 + conv_integer(s_cursor_ch2_delay1) - 10 and
						conv_integer(mouse_x) < x0+nx/2*dx-1024 + conv_integer(s_cursor_ch2_delay1) + 10 then
						v_adjust_ch2_delay1 := '1';
					elsif conv_integer(mouse_y) > y0+ny*dy and conv_integer(mouse_y) <= y0+ny*dy+20 and
						conv_integer(mouse_x) > x0+nx/2*dx-1024 + conv_integer(s_cursor_ch2_delay2) - 10 and
						conv_integer(mouse_x) < x0+nx/2*dx-1024 + conv_integer(s_cursor_ch2_delay2) + 10 then
						v_adjust_ch2_delay2 := '1';
					end if;
				end if;
			end if;

			if s_cursor_ch1 = '1' then										-- channel 1
				if v_adjust_ch1_level1 = '1' then
					if conv_integer(mouse_y) < y0 then 
						s_cursor_ch1_level1 <= conv_std_logic_vector(512 + ny/2*dy, 10);
					elsif conv_integer(mouse_y) > y0+ny*dy then
						s_cursor_ch1_level1 <= conv_std_logic_vector(512 - ny/2*dy, 10);
					else
						s_cursor_ch1_level1 <= conv_std_logic_vector(y0+ny/2*dy+512 - conv_integer(mouse_y), 10);
					end if;
				else
					if s_event(36)(1) = '1' then							-- voltage level 1
						s_cursor_ch1_level1 <= s_cursor_ch1_level1 - s_mouse_scroll(9 downto 0);
					end if;
				end if;

				if v_adjust_ch1_level2 = '1' then
					if conv_integer(mouse_y) < y0 then 
						s_cursor_ch1_level2 <= conv_std_logic_vector(512 + ny/2*dy, 10);
					elsif conv_integer(mouse_y) > y0+ny*dy then
						s_cursor_ch1_level2 <= conv_std_logic_vector(512 - ny/2*dy, 10);
					else
						s_cursor_ch1_level2 <= conv_std_logic_vector(y0+ny/2*dy+512 - conv_integer(mouse_y), 10);
					end if;
				else
					if s_event(37)(1) = '1' then							-- voltage level 2
						s_cursor_ch1_level2 <= s_cursor_ch1_level2 - s_mouse_scroll(9 downto 0);
					end if;
				end if;

				if v_adjust_ch1_delay1 = '1' then
					if conv_integer(mouse_x) < x0 then 
						s_cursor_ch1_delay1 <= conv_std_logic_vector(1024 - nx/2*dx, 11);
					elsif conv_integer(mouse_x) > x0+nx*dx then
						s_cursor_ch1_delay1 <= conv_std_logic_vector(1024 - nx/2*dx + nx*dx, 11);	-- nx can be either even or odd
					else
						s_cursor_ch1_delay1 <= conv_std_logic_vector(conv_integer(mouse_x) - (x0+nx/2*dx-1024), 11);
					end if;
				else
					if s_event(38)(1) = '1' then							-- delay 1
						s_cursor_ch1_delay1 <= s_cursor_ch1_delay1 - s_mouse_scroll(10 downto 0);
					end if;
				end if;

				if v_adjust_ch1_delay2 = '1' then
					if conv_integer(mouse_x) < x0 then 
						s_cursor_ch1_delay2 <= conv_std_logic_vector(1024 - nx/2*dx, 11);
					elsif conv_integer(mouse_x) > x0+nx*dx then
						s_cursor_ch1_delay2 <= conv_std_logic_vector(1024 - nx/2*dx + nx*dx, 11);	-- nx can be either even or odd
					else
						s_cursor_ch1_delay2 <= conv_std_logic_vector(conv_integer(mouse_x) - (x0+nx/2*dx-1024), 11);
					end if;
				else
					if s_event(39)(1) = '1' then							-- delay 2
						s_cursor_ch1_delay2 <= s_cursor_ch1_delay2 - s_mouse_scroll(10 downto 0);
					end if;
				end if;
			end if;

			if s_cursor_ch2 = '1' then										-- channel 2
				if v_adjust_ch2_level1 = '1' then
					if conv_integer(mouse_y) < y0 then 
						s_cursor_ch2_level1 <= conv_std_logic_vector(512 + ny/2*dy, 10);
					elsif conv_integer(mouse_y) > y0+ny*dy then
						s_cursor_ch2_level1 <= conv_std_logic_vector(512 - ny/2*dy, 10);
					else
						s_cursor_ch2_level1 <= conv_std_logic_vector(y0+ny/2*dy+512 - conv_integer(mouse_y), 10);
					end if;
				else
					if s_event(36)(1) = '1' then							-- voltage level 1
						s_cursor_ch2_level1 <= s_cursor_ch2_level1 - s_mouse_scroll(9 downto 0);
					end if;
				end if;

				if v_adjust_ch2_level2 = '1' then
					if conv_integer(mouse_y) < y0 then 
						s_cursor_ch2_level2 <= conv_std_logic_vector(512 + ny/2*dy, 10);
					elsif conv_integer(mouse_y) > y0+ny*dy then
						s_cursor_ch2_level2 <= conv_std_logic_vector(512 - ny/2*dy, 10);
					else
						s_cursor_ch2_level2 <= conv_std_logic_vector(y0+ny/2*dy+512 - conv_integer(mouse_y), 10);
					end if;
				else
					if s_event(37)(1) = '1' then							-- voltage level 2
						s_cursor_ch2_level2 <= s_cursor_ch2_level2 - s_mouse_scroll(9 downto 0);
					end if;
				end if;

				if v_adjust_ch2_delay1 = '1' then
					if conv_integer(mouse_x) < x0 then 
						s_cursor_ch2_delay1 <= conv_std_logic_vector(1024 - nx/2*dx, 11);
					elsif conv_integer(mouse_x) > x0+nx*dx then
						s_cursor_ch2_delay1 <= conv_std_logic_vector(1024 - nx/2*dx + nx*dx, 11);	-- nx can be either even or odd
					else
						s_cursor_ch2_delay1 <= conv_std_logic_vector(conv_integer(mouse_x) - (x0+nx/2*dx-1024), 11);
					end if;
				else
					if s_event(38)(1) = '1' then							-- delay 1
						s_cursor_ch2_delay1 <= s_cursor_ch2_delay1 - s_mouse_scroll(10 downto 0);
					end if;
				end if;

				if v_adjust_ch2_delay2 = '1' then
					if conv_integer(mouse_x) < x0 then 
						s_cursor_ch2_delay2 <= conv_std_logic_vector(1024 - nx/2*dx, 11);
					elsif conv_integer(mouse_x) > x0+nx*dx then
						s_cursor_ch2_delay2 <= conv_std_logic_vector(1024 - nx/2*dx + nx*dx, 11);	-- nx can be either even or odd
					else
						s_cursor_ch2_delay2 <= conv_std_logic_vector(conv_integer(mouse_x) - (x0+nx/2*dx-1024), 11);
					end if;
				else
					if s_event(39)(1) = '1' then							-- delay 2
						s_cursor_ch2_delay2 <= s_cursor_ch2_delay2 - s_mouse_scroll(10 downto 0);
					end if;
				end if;
			end if;

		end if;
	end process;

	cursor_ch1 <= s_cursor_ch1;
	cursor_ch2 <= s_cursor_ch2;
	s_color(34) <= "010" when s_cursor_ch1 = '1' else "111";
	s_color(35) <= "100" when s_cursor_ch2 = '1' else "111";

	s_color(36) <= "010" when s_target(36) = '1' and s_cursor_ch1 = '1' else
						"100" when s_target(36) = '1' and s_cursor_ch2 = '1' else
						"111";

	s_color(37) <= "010" when s_target(37) = '1' and s_cursor_ch1 = '1' else
						"100" when s_target(37) = '1' and s_cursor_ch2 = '1' else
						"111";

	s_color(38) <= "010" when s_target(38) = '1' and s_cursor_ch1 = '1' else
						"100" when s_target(38) = '1' and s_cursor_ch2 = '1' else
						"111";

	s_color(39) <= "010" when s_target(39) = '1' and s_cursor_ch1 = '1' else
						"100" when s_target(39) = '1' and s_cursor_ch2 = '1' else
						"111";

	cursor_en <= s_cursor_ch1 or s_cursor_ch2;
	cursor_level1 <= s_cursor_ch1_level1 when s_cursor_ch1 = '1' else s_cursor_ch2_level1;
	cursor_level2 <= s_cursor_ch1_level2 when s_cursor_ch1 = '1' else s_cursor_ch2_level2;
	cursor_delay1 <= s_cursor_ch1_delay1 when s_cursor_ch1 = '1' else s_cursor_ch2_delay1;
	cursor_delay2 <= s_cursor_ch1_delay2 when s_cursor_ch1 = '1' else s_cursor_ch2_delay2;

-------------------------------------------------------------------------
-- Write buttons text and colors to txt RAM
-------------------------------------------------------------------------

	write_to_txt_ram : process(clk, reset)
	variable v_txt_addr : std_logic_vector(12 downto 0);
	variable v_button_idx : integer range 0 to c_button_count-1;
	variable v_char_cnt : integer range 0 to 63;
	variable v_color : std_logic_vector(2 downto 0);
	begin
		if reset = '1' then
			v_txt_addr := (others => '0');
			v_color := (others => '0');
			v_button_idx := 0;
			v_char_cnt := 0;
			txt_ram_wr <= '0';
			txt_color_ram_wr <= '0';
			txt_ram_addr <= (others => '0');
			txt_ram_data <= (others => '0');
			txt_ram_state <= init;
		elsif clk'event and clk = '1' then
			case txt_ram_state is
			when init =>
				txt_ram_state <= wr;
				v_txt_addr := conv_std_logic_vector(c_button_addr(v_button_idx), 13);	-- to
				v_char_cnt := 1;
				v_color := s_color(v_button_idx);
				txt_color_ram_wr <= '0';
				txt_ram_wr <= '0';
			when wr =>
				txt_ram_addr <= v_txt_addr;
				if v_button_idx = 0 and v_char_cnt <= 5 then			-- timebase
					txt_ram_data <= v_color & conv_std_logic_vector(character'pos(timebase(5*conv_integer(s_sampling_div)+v_char_cnt)), 7);
					txt_color_ram_wr <= '1';
					txt_ram_wr <= '1';
				elsif v_button_idx = 9 and v_char_cnt <= 5 then		-- ch1 V/div
					txt_ram_data <= v_color & conv_std_logic_vector(character'pos(vperdiv(5*conv_integer(s_ch1_gain)+v_char_cnt)), 7);
					txt_color_ram_wr <= '1';
					txt_ram_wr <= '1';
				elsif v_button_idx = 15 and v_char_cnt <= 5 then	-- ch2 V/div
					txt_ram_data <= v_color & conv_std_logic_vector(character'pos(vperdiv(5*conv_integer(s_ch2_gain)+v_char_cnt)), 7);
					txt_color_ram_wr <= '1';
					txt_ram_wr <= '1';
				else
					txt_ram_data <= v_color & "0000000";
					txt_color_ram_wr <= '1';
					txt_ram_wr <= '0';
				end if;
				v_txt_addr := v_txt_addr + 1;
				if v_char_cnt = c_button(v_button_idx).dx then
					txt_ram_state <= init;
					if v_button_idx = c_button_count-1 then 
						v_button_idx := 0;
					else
						v_button_idx := v_button_idx + 1;
					end if;
				else
					v_char_cnt := v_char_cnt + 1;
				end if;
			when others =>
				txt_ram_state <= init;
			end case;
		end if;
	end process;


end rtl;



